/***************************************************************************
 $RCSfile: cfg_services.cpp,v $
                             -------------------
    cvs         : $Id: cfg_services.cpp,v 1.7 2003/02/17 04:19:40 aquamaniac Exp $
    begin       : Wed Oct 09 2002
    copyright   : (C) 2002 by Martin Preuss
    email       : martin@libchipcard.de

 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/


#include "chipcard.h"
#include <qgroupbox.h>
#include <qmessagebox.h>
#include <qspinbox.h>
#include <qlineedit.h>
#include <qfiledialog.h>
#include <string.h>

#include "cfg_services.h"
#include "dlgservice.h"


#ifndef trUtf8
# define trUtf8 tr
#endif



ConfigServices::ConfigServices(QWidget* parent,
			       const char* name,
			       WFlags fl)
:ConfigServicesBase(parent,name,fl)
,ConfigPage()
,_changed(false)
{
  // connect all buttons
  QObject::connect((QObject*)but_add,
		   SIGNAL(clicked()),
		   this,
		   SLOT(slotAdd()));
  QObject::connect((QObject*)but_remove,
		   SIGNAL(clicked()),
		   this,
		   SLOT(slotRemove()));
  QObject::connect((QObject*)but_edit,
		   SIGNAL(clicked()),
		   this,
		   SLOT(slotEdit()));
  QObject::connect((QObject*)ed_commands,
		   SIGNAL(textChanged(const QString &)),
		   this,
		   SLOT(slotChanged(const QString &)));
  QObject::connect((QObject*)but_commands,
		   SIGNAL(clicked()),
		   this,
		   SLOT(slotCommands()));
}


ConfigServices::~ConfigServices(){
}


bool ConfigServices::init(CONFIGGROUP *cfg){
  CONFIGGROUP *curr;

  _timeout=Config_GetIntValue(cfg,
			      "timeout",
			      750,0);
  spo_timeout->setValue(_timeout);

  _maxmsg=Config_GetIntValue(cfg,
			     "maxmsg",
			     20,0);
  spo_maxrequests->setValue(_maxmsg);

  _commands=Config_GetValue(cfg,
			    "commands",
			    CHIPCARD_COMMANDS,0);
  ed_commands->setText(_commands.c_str());

  curr=cfg->groups;
  curr=Config_GetGroup(cfg,"service",
		       CONFIGMODE_PATHMUSTEXIST |
		       CONFIGMODE_NAMEMUSTEXIST);
  if (curr==0) {
    fprintf(stderr,"service section not found\n");
    return true;
  }
  curr=curr->groups;
  if (curr==0) {
    fprintf(stderr,"service section is empty\n");
    return true;
  }

  while (curr) {
    string addr;
    int port;
    Service *ts;
    string tp;
    int tpi;

    tp=Config_GetValue(curr,
		       "type",
		       "net",0);
    if (strcasecmp(tp.c_str(),"local")==0) {
      addr="";
      port=0;
      tpi=CFG_SERVICE_TYPE_LOCAL;
    }
    else if (strcasecmp(tp.c_str(),"net")==0) {
      addr=Config_GetValue(curr,
			   "address",
			   "127.0.0.1",0);
      port=Config_GetIntValue(curr,
			      "port",
			      32891,0);
      if (addr.empty())
	tpi=-1;
      else
	tpi=CFG_SERVICE_TYPE_NET;
    }
    else
      tpi=-1;
    if (tpi!=-1) {
      ts=_findService(tpi,addr,port);
      if (ts)
	fprintf(stderr,"Double entry, skipping\n");
      else
	_services.push_back(Service(tpi, addr, port));
    }
    else {
      fprintf(stderr,"Bad service, skipping\n");
    }
    curr=curr->next;
  } // while

  setChanged(false);
  _updateList();
  return true;
}


bool ConfigServices::save(CONFIGGROUP *grp){
  CONFIGGROUP *cgrp;
  CONFIGGROUP *grp2;
  int i=0;
  QString name;
  int rv;
  list<Service>::iterator it;
  int tp;

  Config_ClearGroup(grp);
  rv=Config_SetValue(grp,
		     CONFIGMODE_VARIABLE |
		     CONFIGMODE_NAMECREATE_VARIABLE |
		     CONFIGMODE_OVERWRITE_VARS,
		     "timeout",
		     spo_timeout->text().latin1());
  if (rv) {
    fprintf(stderr,"Error setting value\n");
    return false;
  }

  rv=Config_SetValue(grp,
		     CONFIGMODE_VARIABLE |
		     CONFIGMODE_NAMECREATE_VARIABLE |
		     CONFIGMODE_OVERWRITE_VARS,
		     "maxmsg",
		     spo_maxrequests->text().latin1());
  if (rv) {
    fprintf(stderr,"Error setting value\n");
    return false;
  }

  name=ed_commands->text();
  if (name.isEmpty()) {
    QMessageBox::critical(this,
			  trUtf8("Save Services"),
			  trUtf8("Please specify the path of the "
				 "command files.\n"),
			  QMessageBox::Ok | QMessageBox::Default,
			  QMessageBox::NoButton,
			  QMessageBox::NoButton);
    return false;
  }
  if (!QFile::exists(name+"/rsacard.cmd")) {
    QMessageBox::critical(this,
			  trUtf8("Save Services"),
			  trUtf8("The path you specified does not "
				 "contain command files.\n"),
			  QMessageBox::Ok | QMessageBox::Default,
			  QMessageBox::NoButton,
			  QMessageBox::NoButton);
    return false;
  }

  rv=Config_SetValue(grp,
		     CONFIGMODE_VARIABLE |
		     CONFIGMODE_NAMECREATE_VARIABLE |
		     CONFIGMODE_OVERWRITE_VARS,
		     "commands",
		     name.latin1());
  if (rv) {
    fprintf(stderr,"Error setting value\n");
    QMessageBox::critical(this,
			  trUtf8("Save Services"),
			  trUtf8("Could not set value (1)\n"
				 "This is an internal error, please report to\n"
				 "martin@libchipcard.de"),
			  QMessageBox::Ok | QMessageBox::Default,
			  QMessageBox::NoButton,
			  QMessageBox::NoButton);
    return false;
  }

  grp2=Config_GetGroup(grp,"service",0);
  if (grp2==0) {
    fprintf(stderr,"could not get services section\n");
    QMessageBox::critical(this,
			  trUtf8("Save Services"),
			  trUtf8("Could not create config group (1)\n"
				 "This is an internal error, please report to\n"
				 "martin@libchipcard.de"),
			  QMessageBox::Ok | QMessageBox::Default,
			  QMessageBox::NoButton,
			  QMessageBox::NoButton);
    return false;
  }

  for (it=_services.begin(); it!=_services.end(); it++) {
    name=QString::number(++i);
    cgrp=Config_GetGroup(grp2,
			 name.latin1(),
			 CONFIGMODE_NAMECREATE_GROUP);
    if (!cgrp) {
      QMessageBox::critical(this,
			    trUtf8("Save Services"),
			    trUtf8("Could not create config group (2)\n"
				   "This is an internal error, please report to\n"
				   "martin@libchipcard.de"),
			    QMessageBox::Ok | QMessageBox::Default,
			    QMessageBox::NoButton,
			    QMessageBox::NoButton);
      return false;
    }
    tp=(*it).type();
    switch((*it).type()) {
    case CFG_SERVICE_TYPE_NET:
      rv=Config_SetValue(cgrp,
			 CONFIGMODE_VARIABLE |
			 CONFIGMODE_NAMECREATE_VARIABLE |
			 CONFIGMODE_OVERWRITE_VARS,
			 "type",
			 "net");
      if (rv) {
	fprintf(stderr,"Error setting value\n");
	return false;
      }
      rv=Config_SetValue(cgrp,
			 CONFIGMODE_VARIABLE |
			 CONFIGMODE_NAMECREATE_VARIABLE |
			 CONFIGMODE_OVERWRITE_VARS,
			 "address",
			 (*it).address().c_str());
      if (rv) {
	fprintf(stderr,"Error setting value\n");
	return false;
      }

      rv=Config_SetIntValue(cgrp,
			    CONFIGMODE_VARIABLE |
			    CONFIGMODE_NAMECREATE_VARIABLE |
			    CONFIGMODE_OVERWRITE_VARS,
			    "port",
			    (*it).port());
      if (rv) {
	fprintf(stderr,"Error setting value\n");
	return false;
      }
      break;
    case CFG_SERVICE_TYPE_LOCAL:
      rv=Config_SetValue(cgrp,
			 CONFIGMODE_VARIABLE |
			 CONFIGMODE_NAMECREATE_VARIABLE |
			 CONFIGMODE_OVERWRITE_VARS,
			 "type",
			 "local");
      if (rv) {
	fprintf(stderr,"Error setting value\n");
	return false;
      }
      rv=Config_SetValue(cgrp,
			 CONFIGMODE_VARIABLE |
			 CONFIGMODE_NAMECREATE_VARIABLE |
			 CONFIGMODE_OVERWRITE_VARS,
			 "address",
			 "/tmp/chipcardd-socket");
      if (rv) {
	fprintf(stderr,"Error setting value\n");
	return false;
      }
      break;
    default:
      break;
    } // switch

  } // for
  setChanged(false);
  return true;
}


bool ConfigServices::changed(){
    return _changed;
}


void ConfigServices::_updateList() {
  list<Service>::iterator it;
  QListViewItem *item;

  lv_services->clear();
  for (it=_services.begin(); it!=_services.end(); it++) {
    switch ((*it).type()) {
    case CFG_SERVICE_TYPE_NET:
      item=new QListViewItem(lv_services,
			     (*it).address().c_str(),
			     QString::number((*it).port()),
			     tr("Network"));
      break;
    case CFG_SERVICE_TYPE_LOCAL:
      item=new QListViewItem(lv_services,
			     "--",
			     "--",
			     tr("Local"));
      break;
    default:
      break;
    } // switch
  } // for
}


ConfigServices::Service *ConfigServices::_findService(int tpi,
						      const string &addr,
						      unsigned int port) {
  list<Service>::iterator it;

  for (it=_services.begin(); it!=_services.end(); it++) {
    if (tpi==CFG_SERVICE_TYPE_LOCAL && (*it).type()==tpi)
      return &(*it);
    if ((*it).address()==addr && (*it).port()==port && (*it).type()==tpi)
      return &(*it);
  }
  return 0;
}


bool ConfigServices::_removeService(int tpi,
				    const string &addr, unsigned int port) {
  list<Service>::iterator it;

  for (it=_services.begin(); it!=_services.end(); it++) {
    if ((*it).address()==addr && (*it).port()==port && (*it).type()==tpi) {
      _services.erase(it);
      return true;
    }
  } // for
  return false;
}


void ConfigServices::setChanged(bool ch){
  _changed=ch;
  //but_save->setEnabled(ch);
}


ConfigServices::Service *ConfigServices::_currentService() {
  QListViewItem *item;
  int port;
  bool ok;

  item=lv_services->currentItem();
  if (!item)
    return 0;
  if (item->text(2)==tr("Network")) {
    port=item->text(1).toInt(&ok);
    if (!ok) {
      fprintf(stderr,"Port must be a number !\n");
      return 0;
    }
    return _findService(CFG_SERVICE_TYPE_NET, item->text(0).latin1(), port);
  }
  else {
    return _findService(CFG_SERVICE_TYPE_LOCAL,"",0);
  }
}


void ConfigServices::slotEdit(){
  CTPointer<dlgService> dlgs;
  int answer;
  Service *ts;

  fprintf(stderr,"Editing.\n");
  // get currently selected service
  ts=_currentService();
  if (!ts) {
    QMessageBox::critical(this,
			  trUtf8("Edit Service"),
			  trUtf8("You must first select a service"),
			  QMessageBox::Ok | QMessageBox::Default,
			  QMessageBox::NoButton,
			  QMessageBox::NoButton);
    return;
  }

  dlgs=new dlgService(ts->type(),ts->address(), ts->port(),this,0,true);
  if (!dlgs.ref().init())
    return;
  answer=dlgs.ref().exec();
  if (answer==QDialog::Accepted) {
    string addr;
    unsigned int port;
    int bak;
    Service *tmps;
    int tp;

    if (!dlgs.ref().service(tp, addr, port)) {
      QMessageBox::critical(this,
			    trUtf8("Edit Service"),
			    trUtf8("Could not get service, unchanged"),
			    QMessageBox::Ok | QMessageBox::Default,
			    QMessageBox::NoButton,
			    QMessageBox::NoButton);
      return;
    }
    bak=ts->type();
    ts->setType(-1);
    tmps=_findService(tp, addr, port);
    ts->setType(bak);
    if (tmps) {
      QMessageBox::critical(this,
			    trUtf8("Edit Service"),
			    trUtf8("This service already exists, unchanged"),
			    QMessageBox::Ok | QMessageBox::Default,
			    QMessageBox::NoButton,
			    QMessageBox::NoButton);
      return;
    }
    ts->setType(tp);
    ts->setAddress(addr);
    ts->setPort(port);
    setChanged(true);
  }
  _updateList();
}


void ConfigServices::slotAdd(){
  Service *ts;

  ts=_findService(CFG_SERVICE_TYPE_NET,"<none>",0);
  if (ts) {
    QMessageBox::critical(this,
			  trUtf8("Edit Service"),
			  trUtf8("This service already exists, unchanged"),
			  QMessageBox::Ok | QMessageBox::Default,
			  QMessageBox::NoButton,
			  QMessageBox::NoButton);
    return;
  }
  else {
    _services.push_back(Service(CFG_SERVICE_TYPE_NET,"<none>",0));
    setChanged(true);
    _updateList();
  }
}


void ConfigServices::slotRemove(){
  Service *ts;

  fprintf(stderr,"Removing.\n");
  // get currently selected service
  ts=_currentService();
  if (!ts) {
    QMessageBox::critical(this,
			  trUtf8("Remove Service"),
			  trUtf8("You must first select a service"),
			  QMessageBox::Ok | QMessageBox::Default,
			  QMessageBox::NoButton,
			  QMessageBox::NoButton);
    return;
  }
  if (!_removeService(ts->type(),ts->address(), ts->port())) {
    QMessageBox::critical(this,
			  trUtf8("Remove Service"),
			  trUtf8("Could not remove the service"),
			  QMessageBox::Ok | QMessageBox::Default,
			  QMessageBox::NoButton,
			  QMessageBox::NoButton);
    return;
  }
  setChanged(true);
  _updateList();
}


void ConfigServices::slotChanged(const QString &t){
  _changed=true;
}

void ConfigServices::slotChanged(int s){
  _changed=true;
}


void ConfigServices::slotCommands() {
  // get file name
  QString f=QFileDialog::getExistingDirectory(ed_commands->text(),
					      this,
					      0,
					      trUtf8("Select Commands Path"));
  if (!f)
    return;
  ed_commands->setText(f);
}






