/***************************************************************************
 $RCSfile: cfg_services.h,v $
                             -------------------
    cvs         : $Id: cfg_services.h,v 1.4 2003/02/17 04:19:40 aquamaniac Exp $
    begin       : Wed Oct 09 2002
    copyright   : (C) 2002 by Martin Preuss
    email       : martin@libchipcard.de

 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef CFG_SERVICES_BASE_H
#define CFG_SERVICES_BASE_H

#include <qlistview.h>
#include <chipcard.h>

#include <list>
using namespace std;

#include "configpage.h"
#include "cfg_services_base.h"

/* Please note that these definitions correspond with the order they
 * appear in the combobox !
 */
#define CFG_SERVICE_TYPE_NET   0
#define CFG_SERVICE_TYPE_LOCAL 1


class ConfigServices : public ConfigServicesBase, public ConfigPage{
  Q_OBJECT

public:
  class Service {
  private:
    string _address;
    unsigned int _port;
    int _type;
  public:
    Service(int tp,
	    const string &a,
	    unsigned int p):_address(a),_port(p),_type(tp) {
    };
    ~Service(){};
    const string &address() const { return _address;};
    unsigned int port() const { return _port;};
    void setAddress(const string &a) { _address=a;};
    void setPort(unsigned int p) { _port=p;};
    int type() const { return _type;};
    void setType(int t) { _type=t;};
  };

private:
  bool _changed;
  list<Service> _services;
  int _timeout;
  int _maxmsg;
  string _commands;

  void _updateList();
  bool _removeService(int tpi, const string &addr, unsigned int port);
  Service *_currentService();
  Service *_findService(int tpi, const string &addr, unsigned int port);

public:
  ConfigServices(QWidget* parent=0, const char* name=0, WFlags fl=0);
  ~ConfigServices();

  virtual bool init(CONFIGGROUP *cfg);
  virtual bool save(CONFIGGROUP *cfg);
  virtual bool changed();
  void setChanged(bool ch);

protected slots:
  void slotAdd();
  void slotRemove();
  void slotEdit();
  void slotChanged(const QString &t);
  void slotChanged(int s);
  void slotCommands();
};



#endif // CFG_SERVICES_BASE_H




