/***************************************************************************
  $RCSfile: cfg_terminals.cpp,v $
                             -------------------
    cvs         : $Id: cfg_terminals.cpp,v 1.7 2003/02/18 00:42:12 aquamaniac Exp $
    begin       : Sat Mar 16 2002
    copyright   : (C) 2002 by Martin Preuss
    email       : martin@libchipcard.de


 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include <qlistview.h>
#include <qmessagebox.h>
#include <qpushbutton.h>
#include <qcheckbox.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qspinbox.h>
#include <qcombobox.h>

#include "dlgterm.h"
#include "cfg_terminals.h"
#include "cfg_services.h"
#include "dlgaccess.h"
#include <assert.h>

#ifndef trUtf8
# define trUtf8 tr
#endif



ConfigTerminals::ConfigTerminals(QWidget* parent,
				 const char* name,
				 WFlags fl)
  :ConfigTerminalsBase(parent,name,fl)
  ,ConfigPage()
  ,_changed(false)
  ,_type(0)
  ,_readers(0)
  ,_drivers(0)
{
    // connect all buttons
    QObject::connect((QObject*)but_add,
                     SIGNAL(clicked()),
                     this,
                     SLOT(slotAdd()));
    QObject::connect((QObject*)but_remove,
                     SIGNAL(clicked()),
                     this,
                     SLOT(slotRemove()));
    QObject::connect((QObject*)but_edit,
                     SIGNAL(clicked()),
                     this,
                     SLOT(slotEdit()));
    QObject::connect((QObject*)but_copy,
                     SIGNAL(clicked()),
                     this,
		     SLOT(slotCopy()));
    QObject::connect((QObject*)ed_address,
		     SIGNAL(textChanged(const QString &)),
		     this,
		     SLOT(slotChanged(const QString &)));
    QObject::connect((QObject*)ed_port,
		     SIGNAL(textChanged(const QString &)),
		     this,
		     SLOT(slotChanged(const QString &)));
    QObject::connect((QObject*)spo_timeout,
		     SIGNAL(valueChanged(const QString &)),
		     this,
		     SLOT(slotChanged(const QString &)));
    QObject::connect((QObject*)spo_maxrequests,
		     SIGNAL(valueChanged(const QString &)),
		     this,
		     SLOT(slotChanged(const QString &)));
    QObject::connect((QObject*)cbx_enable,
		     SIGNAL(stateChanged(int)),
		     this,
		     SLOT(slotEnableChanged(int)));
    QObject::connect((QObject*)cbo_type,
		     SIGNAL(activated(int)),
		     this,
		     SLOT(slotTypeChanged(int)));
    QObject::connect((QObject*)ed_access,
		     SIGNAL(textChanged(const QString &)),
		     this,
		     SLOT(slotChanged(const QString &)));
    QObject::connect((QObject*)but_access,
		     SIGNAL(clicked()),
		     this,
		     SLOT(slotRights()));
    //but_save->setEnabled(false);
}


ConfigTerminals::~ConfigTerminals(){
  if (_drivers)
    Config_free(_drivers);
}


bool ConfigTerminals::init(CONFIGGROUP *cfg) {
  CONFIGGROUP *curr;
  const char *tp;
  unsigned int rights;
  const char *tmp;

  // read driver descriptions
  _drivers=Config_new();
  if (CTCore_ReadDriverDescriptions(CHIPCARD_DRIVERS,
				    _drivers)) {
    QMessageBox::critical(this,
			  trUtf8("Loading Driver Descriptions"),
			  trUtf8("Could not load driver descriptions.\n"
				 "Please make sure that the \"*.dsc\" file\n"
				 "are properly installed."),
			  QMessageBox::Ok | QMessageBox::Default,
			  QMessageBox::NoButton,
			  QMessageBox::NoButton);
    return false;
  }

  // general settings
  _timeout=Config_GetIntValue(cfg,
			      "timeout",
			      750,0);
  spo_timeout->setValue(_timeout);

  _maxmsg=Config_GetIntValue(cfg,
			     "maxmsg",
			     20,0);
  spo_maxrequests->setValue(_maxmsg);


  _enabled=Config_GetIntValue(cfg,
			      "enabled",
			      0,0);
  cbx_enable->setChecked(_enabled);

  tmp=Config_GetValue(cfg,
		      "access",
		      "0777",0);
  if (sscanf(tmp,"%od",&rights)!=1) {
    fprintf(stderr,"Bad access value, using default.\n");
    rights=0777;
  }
  ed_access->setText(QString::number(rights,8));

  tp=Config_GetValue(cfg,
		     "type",
		     "net",0);
  if (strcasecmp(tp,"net")==0){
      cbo_type->setCurrentItem(CFG_SERVICE_TYPE_NET);
      _address=Config_GetValue(cfg,
			       "address",
			       "127.0.0.1",0);
      ed_address->setText(_address.c_str());

      _port=Config_GetIntValue(cfg,
			       "port",
			       32891,0);
      ed_port->setText(QString::number(_port));
      _type=CFG_SERVICE_TYPE_NET;
  }
  else {
      cbo_type->setCurrentItem(CFG_SERVICE_TYPE_LOCAL);
      _type=CFG_SERVICE_TYPE_LOCAL;
      ed_address->setText("");
      ed_port->setText("");
  }

  // read readers
  curr=Config_GetGroup(cfg,"reader",
		       CONFIGMODE_PATHMUSTEXIST |
		       CONFIGMODE_NAMEMUSTEXIST);
  if (curr) {
      curr=curr->groups;
      while (curr) {
	  CTREADERDESCRIPTION *ds;

	  ds=CTCore_ReadReaderDescr(curr);
	  if (!ds) {
	      fprintf(stderr,"error in reader section \"%s\"\n",
		      curr->name);
	  }
	  else {
	      fprintf(stderr,"Adding reader \"%s\"\n", ds->name);
	      CTCore_ReaderDescr_Add(ds, &_readers);
	  }
	  curr=curr->next;
      } // while
  }

  adjustToMode(_enabled, _type==CFG_SERVICE_TYPE_NET);

  updateList();

  _changed=false;
  return true;
}


void ConfigTerminals::updateList() {
  QListViewItem *item;
  CTREADERDESCRIPTION *ds;

  list_readers->clear();
  ds=_readers;

  while(ds) {
    item=new QListViewItem(list_readers,
			   ds->name,
			   ds->typeName,
			   ds->portName);
    ds=ds->next;
  } // while
}


CTREADERDESCRIPTION *ConfigTerminals::_findReader(const string &r) {
  CTREADERDESCRIPTION *curr;

  curr=_readers;
  while(curr) {
    if (r==curr->name)
      return curr;
    curr=curr->next;
  } // while
  return 0;
}


bool ConfigTerminals::_removeReader(CTREADERDESCRIPTION *reader) {
  if (reader==0)
    return false;
  CTCore_ReaderDescr_Remove(reader, &_readers);
  return true;
}


string ConfigTerminals::_makeNextName(const string &name) {
  unsigned int i;
  int num;
  string body;
  char buffer[16];

  num=0;
  body=name;

  // first get the number at the end of this name
  for (i=name.length()-1;
       i>=0;
       i--) {
    // is there still a digit ?
    if (name.at(i)<'0' || name.at(i)>'9')
      // no
      break;
  } // for

  // valid pos ?
  if (i>=0 && i<name.length()-1){
    // yes, read the number, create the body
    num=atoi((name.substr(i+1)).c_str());
    body=name.substr(0,i+1);
  }

  // increment number
  num++;

  // create new name and return
  snprintf(buffer,sizeof(buffer),"%d",num);
  body+=buffer;
  return body;
}



CTREADERDESCRIPTION *ConfigTerminals::currentReader() {
  QListViewItem *item;

  item=list_readers->currentItem();
  if (!item)
    return 0;
  return _findReader(item->text(0).latin1());
}


void ConfigTerminals::setChanged(bool ch){
  _changed=ch;
  //but_save->setEnabled(ch);
}



void ConfigTerminals::slotAdd(){
  string name;
  int i;
  CTREADERDESCRIPTION *ds;

  // try to create a unique name 20 times
  name="term";
  i=20;
  while(i) {
    name=_makeNextName(name);
    if (!(_findReader(name)))
      break;
    i--;
  }

  if (!i) {
    // could not find a unique name within 10 tries
    QMessageBox::critical(this,
			  trUtf8("Add reader"),
			  trUtf8("Could not create a unique name\n"
				 "This is an internal error, please report to\n"
				 "martin@libchipcard.de"),
			  QMessageBox::Ok | QMessageBox::Default,
			  QMessageBox::NoButton,
			  QMessageBox::NoButton);
    return;
  }

  // init new struct
  ds=CTCore_ReaderDescr_new();
  if ((name.length()+1)<sizeof(ds->name))
    strcpy(ds->name,name.c_str());
  ds->driverType=DriverTypeCTAPI;
  strcpy(ds->typeName,"towitoko");
  strcpy(ds->portName,"COM2");
  // add new struct to list
  fprintf(stderr,"Adding reader \"%s\"\n", ds->name);
  CTCore_ReaderDescr_Add(ds,&_readers);

  setChanged(true);
  updateList();
}


void ConfigTerminals::slotRemove(){
  CTREADERDESCRIPTION *ts;

  // get currently selected reader
  ts=currentReader();
  if (!ts) {
    QMessageBox::critical(this,
			  trUtf8("Remove Reader"),
			  trUtf8("You must first select a device"),
			  QMessageBox::Ok | QMessageBox::Default,
			  QMessageBox::NoButton,
			  QMessageBox::NoButton);
    return;
  }

  // ask the user
  if (QMessageBox::warning(this,
			   trUtf8("Remove Reader"),
			   trUtf8("Really remove this device ?"),
			   QMessageBox::Ok | QMessageBox::Default,
			   QMessageBox::Abort | QMessageBox::Escape,
			   QMessageBox::NoButton)!=QMessageBox::Ok)
    return;

  // remove the terminal
  if (!_removeReader(ts))
    QMessageBox::critical(this,
			  trUtf8("Remove Reader"),
			  trUtf8("Could not remove reader"),
			  QMessageBox::Ok | QMessageBox::Default,
			  QMessageBox::NoButton,
			  QMessageBox::NoButton);
  setChanged(true);
  updateList();
}


void ConfigTerminals::slotEdit(){
  CTPointer<dlgTerm> dlgt;
  CTREADERDESCRIPTION *ts;
  int answer;

  // get currently selected reader
  ts=currentReader();
  if (!ts) {
    QMessageBox::critical(this,
			  trUtf8("Edit Reader"),
			  trUtf8("You must first select a device"),
			  QMessageBox::Ok | QMessageBox::Default,
			  QMessageBox::NoButton,
			  QMessageBox::NoButton);
    return;
  }
  fprintf(stderr,"Editing reader \"%s\"\n",ts->name);
  dlgt=new dlgTerm(_readers,ts,_drivers, this,0,true);
  if (!dlgt.ref().init())
    return;
  answer=dlgt.ref().exec();
  if (answer==QDialog::Accepted) {
    fprintf(stderr,"Saving reader data\n");
    if (!dlgt.ref().saveReader(ts)) {
      QMessageBox::critical(this,
			    trUtf8("Edit Reader"),
			    trUtf8("Could not save the edited reader data"),
			    QMessageBox::Ok | QMessageBox::Default,
			    QMessageBox::NoButton,
			    QMessageBox::NoButton);
    }
    fprintf(stderr,"Edited reader \"%s\"\n",ts->name);
    setChanged(true);
  }
  fprintf(stderr,"Edit reader finished.\n");
  updateList();
}


void ConfigTerminals::slotCopy(){
  CTREADERDESCRIPTION *ts;
  CTREADERDESCRIPTION *ts2;
  string name;
  int i;

  // get currently selected reader
  ts=currentReader();
  if (!ts) {
    QMessageBox::critical(this,
			  trUtf8("Copy reader"),
			  trUtf8("You must first select a device"),
			  QMessageBox::Ok | QMessageBox::Default,
			  QMessageBox::NoButton,
			  QMessageBox::NoButton);
    return;
  }
  // copy the definition
  ts2=CTCore_ReaderDescr_Duplicate(ts);
  name=ts2->name;

  // try to create a unique name 20 times
  i=20;
  while(i) {
    name=_makeNextName(name);
    if (!(_findReader(name)))
      break;
    i--;
  }

  // could not find a unique name within 10 tries
  if (!i) {
    QMessageBox::critical(this,
			  trUtf8("Copy reader"),
			  trUtf8("Could not create a unique name\n"
				 "This is an internal error, please report to\n"
				 "martin@libchipcard.de"),
			  QMessageBox::Ok | QMessageBox::Default,
			  QMessageBox::NoButton,
			  QMessageBox::NoButton);
    CTCore_ReaderDescr_free(ts2);
    return;
  }
  if (name.length()>sizeof(ts2->name)) {
    QMessageBox::critical(this,
			  trUtf8("Copy reader"),
			  trUtf8("Name is too long\n"
				 "This is an internal error, please report to\n"
				 "martin@libchipcard.de"),
			  QMessageBox::Ok | QMessageBox::Default,
			  QMessageBox::NoButton,
			  QMessageBox::NoButton);
    CTCore_ReaderDescr_free(ts2);
    return;
  }
  strcpy(ts2->name, name.c_str());
  fprintf(stderr,"Adding reader \"%s\"\n", ts2->name);
  CTCore_ReaderDescr_Add(ts2, &_readers);
  setChanged(true);
  updateList();
}


void ConfigTerminals::slotChanged(const QString &t){
  _changed=true;
}


void ConfigTerminals::slotChanged(int s){
  _changed=true;
}


void ConfigTerminals::slotTypeChanged(int s){
  _changed=true;
  _type=s;
  adjustToMode(_enabled, _type==CFG_SERVICE_TYPE_NET);
}


void ConfigTerminals::adjustToMode(bool en, bool net) {
  ed_address->setEnabled(en && net);
  spo_timeout->setEnabled(en);
  spo_maxrequests->setEnabled(en);
  ed_port->setEnabled(en && net);
  but_access->setEnabled(en && !net);
  ed_access->setEnabled(en && !net);
}


void ConfigTerminals::slotEnableChanged(int s){
  _changed=true;
  _enabled=s;
  cbo_type->setEnabled(s);
  adjustToMode(_enabled, _type==CFG_SERVICE_TYPE_NET);
}


bool ConfigTerminals::save(CONFIGGROUP *grp){
  CONFIGGROUP *cgrp;
  CONFIGGROUP *grp2;
  CTREADERDESCRIPTION *rs;
  int i=0;
  QString name;
  int rv;
  int tp;

  Config_ClearGroup(grp);
  tp=cbo_type->currentItem();
  if (tp==CFG_SERVICE_TYPE_NET) {
    rv=Config_SetValue(grp,
		       CONFIGMODE_VARIABLE |
		       CONFIGMODE_NAMECREATE_VARIABLE |
		       CONFIGMODE_OVERWRITE_VARS,
		       "type",
		       "net");
    if (rv) {
      fprintf(stderr,"Error setting value\n");
      return false;
    }

    rv=Config_SetValue(grp,
		       CONFIGMODE_VARIABLE |
		       CONFIGMODE_NAMECREATE_VARIABLE |
		       CONFIGMODE_OVERWRITE_VARS,
		       "access",
		       ed_access->text().latin1());
    if (rv) {
      fprintf(stderr,"Error setting value\n");
      return false;
    }

    rv=Config_SetValue(grp,
		       CONFIGMODE_VARIABLE |
		       CONFIGMODE_NAMECREATE_VARIABLE |
		       CONFIGMODE_OVERWRITE_VARS,
		       "address",
		       ed_address->text().latin1());
    if (rv) {
      fprintf(stderr,"Error setting value\n");
      return false;
    }

    rv=Config_SetValue(grp,
		       CONFIGMODE_VARIABLE |
		       CONFIGMODE_NAMECREATE_VARIABLE |
		       CONFIGMODE_OVERWRITE_VARS,
		       "port",
		       ed_port->text().latin1());
    if (rv) {
      fprintf(stderr,"Error setting value\n");
      return false;
    }
  }
  else {
    rv=Config_SetValue(grp,
		       CONFIGMODE_VARIABLE |
		       CONFIGMODE_NAMECREATE_VARIABLE |
		       CONFIGMODE_OVERWRITE_VARS,
		       "type",
		       "local");
    if (rv) {
      fprintf(stderr,"Error setting value\n");
      return false;
    }
    rv=Config_SetValue(grp,
		       CONFIGMODE_VARIABLE |
		       CONFIGMODE_NAMECREATE_VARIABLE |
		       CONFIGMODE_OVERWRITE_VARS,
		       "address",
		       "/tmp/chipcardd-socket");
    if (rv) {
      fprintf(stderr,"Error setting value\n");
      return false;
    }
  }
  rv=Config_SetValue(grp,
		     CONFIGMODE_VARIABLE |
		     CONFIGMODE_NAMECREATE_VARIABLE |
		     CONFIGMODE_OVERWRITE_VARS,
		     "timeout",
		     spo_timeout->text().latin1());
  if (rv) {
    fprintf(stderr,"Error setting value\n");
    return false;
  }

  rv=Config_SetValue(grp,
		     CONFIGMODE_VARIABLE |
		     CONFIGMODE_NAMECREATE_VARIABLE |
		     CONFIGMODE_OVERWRITE_VARS,
		     "maxmsg",
		     spo_maxrequests->text().latin1());
  if (rv) {
    fprintf(stderr,"Error setting value\n");
    return false;
  }

  rv=Config_SetIntValue(grp,
			CONFIGMODE_VARIABLE |
			CONFIGMODE_NAMECREATE_VARIABLE |
			CONFIGMODE_OVERWRITE_VARS,
			"enabled",
			cbx_enable->isChecked());
  if (rv) {
    fprintf(stderr,"Error setting value\n");
    return false;
  }

  grp2=Config_GetGroup(grp,"reader",0);
  if (grp2==0) {
    fprintf(stderr,"could not get reader section\n");
    QMessageBox::critical(this,
			  trUtf8("Save Readers"),
			  trUtf8("Could not create config group (1)\n"
				 "This is an internal error, please report to\n"
				 "martin@libchipcard.de"),
			  QMessageBox::Ok | QMessageBox::Default,
			  QMessageBox::NoButton,
			  QMessageBox::NoButton);
    return false;
  }

  rs=_readers;
  while (rs) {
    name=QString::number(++i);
    cgrp=Config_GetGroup(grp2,
			 name.latin1(),
			 CONFIGMODE_NAMECREATE_GROUP);
    if (!cgrp) {
      QMessageBox::critical(this,
			    trUtf8("Save Readers"),
			    trUtf8("Could not create config group (2)\n"
				   "This is an internal error, please report to\n"
				   "martin@libchipcard.de"),
			    QMessageBox::Ok | QMessageBox::Default,
			    QMessageBox::NoButton,
			    QMessageBox::NoButton);
      return false;
    }
    rv=CTCore_WriteReaderDescr(cgrp, rs);
    if (rv) {
      QMessageBox::critical(this,
			    trUtf8("Save Readers"),
			    trUtf8("Could not write reader description\n"
				   "This is an internal error, please report to\n"
				   "martin@libchipcard.de"),
			    QMessageBox::Ok | QMessageBox::Default,
			    QMessageBox::NoButton,
			    QMessageBox::NoButton);
      return false;
    }
    rs=rs->next;
  } // while
  setChanged(false);
  return true;
}


bool ConfigTerminals::changed(){
  return _changed;
}


void ConfigTerminals::slotRights() {
  unsigned int i, j;
  int rv;
  dlgAccess *dlg;
  bool ok;

  i=ed_access->text().toUInt(&ok,8);
  if (!ok)
    i=0777;
  dlg=new dlgAccess(i,0777,
		    this,
		    "AccessDialog",
		    Qt::WType_TopLevel|Qt::WStyle_ContextHelp);
  dlg->show();
#if QT_GENERATION<3
  rv=dlg->result();
#else
  rv=dlg->exec();
#endif
  j=dlg->rights();
  dlg->hide();
  if (rv==QDialog::Accepted) {
    if (i!=j)
      ed_access->setText(QString::number(j,8));
  }
  delete dlg;
}



