/***************************************************************************
  $RCSfile: cfg_terminals.h,v $
                             -------------------
    cvs         : $Id: cfg_terminals.h,v 1.4 2003/02/18 00:42:12 aquamaniac Exp $
    begin       : Sat Mar 16 2002
    copyright   : (C) 2002 by Martin Preuss
    email       : martin@libchipcard.de


 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef DLGLCSETUP_H
#define DLGLCSETUP_H

#include <string>
#include <list>
using namespace std;

#include <chipcard.h>
#include <chipcard/ctcore_public.h>
#include "cfg_terminals_base.h"
#include "configpage.h"



/**
 *
 */
class ConfigTerminals: public ConfigTerminalsBase, public ConfigPage {
  Q_OBJECT
private:
  bool _changed;
  int _type;
  CTREADERDESCRIPTION *_readers;
  bool _enabled;
  string _address;
  unsigned int _port;
  int _timeout;
  int _maxmsg;
  CONFIGGROUP *_drivers;

  string _makeNextName(const string &name);
  CTREADERDESCRIPTION *_findReader(const string &r);
  bool _removeReader(CTREADERDESCRIPTION *reader);
  int _getReaderPos(const string &r);

public:
  /**
   * Constructor.
   * @param Martin Preuss<martin@libchipcard.de>
   * @param cfgfilename path and name of the configuration file to edit
   */
  ConfigTerminals(QWidget* parent = 0,
		  const char* name = 0,
		  WFlags fl = 0 );
  ~ConfigTerminals();
  virtual bool init(CONFIGGROUP *cfg);
  virtual bool save(CONFIGGROUP *cfg);
  virtual bool changed();

protected:
  CTREADERDESCRIPTION *currentReader();
  void updateList();
  void setChanged(bool ch);
  void adjustToMode(bool en, bool net);

protected slots:
  void slotAdd();
  void slotRemove();
  void slotEdit();
  void slotCopy();
  void slotChanged(const QString &t);
  void slotChanged(int s);
  void slotEnableChanged(int s);
  void slotTypeChanged(int s);
  void slotRights();
};



#endif







