/***************************************************************************
 $RCSfile: configdialog.cpp,v $
                             -------------------
    cvs         : $Id: configdialog.cpp,v 1.5 2003/02/17 04:19:40 aquamaniac Exp $
    begin       : Wed Oct 09 2002
    copyright   : (C) 2002 by Martin Preuss
    email       : martin@libchipcard.de

 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/


#include <qmessagebox.h>

#include "configdialog.h"


#ifndef trUtf8
# define trUtf8 tr
#endif


#define CONFIG_CLIENTFILE CHIPCARDC_CFGFILE
#define CONFIG_SERVERFILE CHIPCARDD_CFGFILE


ConfigDialog::ConfigDialog(QWidget *parent,
			   const char *name,
			   bool modal,
			   WFlags fl)
  :QTabDialog(parent, name, modal, fl)
  ,_cfgClient(0)
  ,_cfgServer(0)
  ,_cfgWelcome(0)
  ,_cfgTerms(0)
  ,_cfgServices(0)
{
}


ConfigDialog::~ConfigDialog(){
  Config_free(_cfgClient);
  Config_free(_cfgServer);
}


bool ConfigDialog::init(){
  int rv1;
  int rv2;

  setMinimumSize(610,480);
  setMaximumSize(610,480);
  resize(610,480);

  _cfgClient=Config_new();
  _cfgServer=Config_new();

  rv1=Config_ReadFile(_cfgServer,
		      CONFIG_SERVERFILE,
		      CONFIGMODE_REMOVE_QUOTES |
		      CONFIGMODE_REMOVE_STARTING_BLANKS |
		      CONFIGMODE_REMOVE_TRAILING_BLANKS |
		      CONFIGMODE_ALLOW_GROUPS);
  rv2=Config_ReadFile(_cfgClient,
		      CONFIG_CLIENTFILE,
		      CONFIGMODE_REMOVE_QUOTES |
		      CONFIGMODE_REMOVE_STARTING_BLANKS |
		      CONFIGMODE_REMOVE_TRAILING_BLANKS |
		      CONFIGMODE_ALLOW_GROUPS);
  if (rv1)
    fprintf(stderr,"Server configuration file not found, creating it.\n");
  if (rv2)
    fprintf(stderr,"Client configuration file not found, creating it.\n");

  // welcome tab
  _cfgWelcome=new ConfigWelcome(this);
  if (!_cfgWelcome->init(_cfgClient))
    return false;
  addTab(_cfgWelcome, tr("Welcome"));
  _cfgWelcome->show();

  // service tab
  _cfgServices=new ConfigServices(this);
  if (!_cfgServices->init(_cfgClient))
    return false;
  addTab(_cfgServices, tr("Client"));
  _cfgServices->show();

  // terminal tab
  _cfgTerms=new ConfigTerminals(this);
  if (!_cfgTerms->init(_cfgServer))
    return false;
  addTab(_cfgTerms, tr("Server"));
  _cfgTerms->show();

  // show some additional buttons
  setCancelButton();
  //setHelpButton();
  setApplyButton();
  //setDefaultButton();

  // connect signals
  QObject::connect(this,
		   SIGNAL(applyButtonPressed()),
		   this,
		   SLOT(slotApply()));
  QObject::connect(this,
		   SIGNAL(helpButtonPressed()),
		   this,
		   SLOT(slotHelp()));
  QObject::connect(this,
		   SIGNAL(defaultButtonPressed()),
		   this,
		   SLOT(slotDefault()));
  return true;
}


void ConfigDialog::slotDefault(){
}


void ConfigDialog::slotApply(){
  fprintf(stderr,"Applying.\n");
  save();
}


void ConfigDialog::slotHelp(){
}


bool ConfigDialog::save() {
  int rv1, rv2;

  if (!_cfgTerms->save(_cfgServer)) {
    fprintf(stderr,"Error saving terminals.\n");
    return false;
  }
  if (!_cfgServices->save(_cfgClient)) {
    fprintf(stderr,"Error saving services.\n");
    return false;
  }
  rv1=Config_WriteFile(_cfgClient,
		       CONFIG_CLIENTFILE,
		       CONFIGMODE_REMOVE_QUOTES |
		       CONFIGMODE_REMOVE_STARTING_BLANKS |
		       CONFIGMODE_REMOVE_TRAILING_BLANKS |
		       CONFIGMODE_ALLOW_GROUPS);
  rv2=Config_WriteFile(_cfgServer,
		       CONFIG_SERVERFILE,
		       CONFIGMODE_REMOVE_QUOTES |
		       CONFIGMODE_REMOVE_STARTING_BLANKS |
		       CONFIGMODE_REMOVE_TRAILING_BLANKS |
		       CONFIGMODE_ALLOW_GROUPS);
  if (rv1 || rv2) {
    QMessageBox::critical(this,
			  trUtf8("KCardSetup"),
			  trUtf8("Could not write configuration."),
			  trUtf8("&Ok"),0,0);
    return false;
  }

  return true;
}


bool ConfigDialog::abort() {
  bool chg=false;

  chg|=_cfgTerms->changed();
  chg|=_cfgServices->changed();

  if (chg) {
    switch(QMessageBox::warning(this,
				trUtf8("KCardSetup"),
				trUtf8("Configuration has changed\n"
				       "Do you want to save it now ?"),
				trUtf8("&Save"),
				trUtf8("&Don't Save"),
				trUtf8("&Cancel"),
				0,2))
      {
      case 0: // save
	// save data
	if (!save())
	  if (QMessageBox::critical(this,
				    trUtf8("KCardSetup"),
				    trUtf8("Could not write configuration\n"
					   "Do you want to exit anyway ?"),
				    trUtf8("&Exit"),
				    trUtf8("&Don't Exit"),
				    0, 1,0)!=0)
	    return false;
	break;

      case 1: // don't save
	break;

      case 2: // cancel
	return false;

      } // switch
  } // if changed

  // close application
  return true;
}


bool ConfigDialog::finished() {
  bool chg=false;

  chg|=_cfgTerms->changed();
  chg|=_cfgServices->changed();

  if (chg) {
    switch(QMessageBox::warning(this,
				trUtf8("KCardSetup"),
				trUtf8("Configuration has changed\n"
				       "Do you want to save it now ?"),
				trUtf8("&Save"),
				trUtf8("&Don't Save"),
				trUtf8("&Cancel"),
				0,2))
      {
      case 0: // save
	// save data
	if (!save())
	  if (QMessageBox::critical(this,
				    trUtf8("KCardSetup"),
				    trUtf8("Could not write configuration\n"
					   "Do you want to exit anyway ?"),
				    trUtf8("&Exit"),
				    trUtf8("&Abort")
				    ,0,1,0)!=0)
	    return false;
	break;

      case 1: // don't save
	break;

      case 2: // cancel
	return false;

      default:
	break;
      } // switch
  } // if _changed

  return true;
}


void ConfigDialog::closeEvent(QCloseEvent *e){
  fprintf(stderr,"closeEvent entered.\n");
  if (abort())
    e->accept();
}


void ConfigDialog::accept(){
  fprintf(stderr,"Accept signalled.\n");
  if (finished())
    QTabDialog::accept();
}


void ConfigDialog::reject(){
  fprintf(stderr,"Reject signalled.\n");
    if (abort())
	QTabDialog::reject();
}





