/***************************************************************************
 $RCSfile: dlgaccess.cpp,v $
 -------------------
 cvs         : $Id: dlgaccess.cpp,v 1.1 2003/02/18 00:42:12 aquamaniac Exp $
 begin       : Mon Feb 17 2003
 copyright   : (C) 2003 by Martin Preuss
 email       : martin@libchipcard.de


 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "dlgaccess.h"
#include <sys/types.h>
#include <sys/stat.h>

#include <qcheckbox.h>


dlgAccess::dlgAccess(unsigned int rights,
		     unsigned int defaultrights,
		     QWidget * parent,
		     const char * name,
		     bool modal,
		     WFlags f)
  :dlgAccessBase(parent, name,modal, f)
  ,_rights(rights)
  ,_default(defaultrights)
{
  QObject::connect((QObject*)but_ok,
		   SIGNAL(clicked()),
		   this,
		   SLOT(slotOk()));
  QObject::connect((QObject*)but_abort,
		   SIGNAL(clicked()),
		   this,
		   SLOT(slotAbort()));
  QObject::connect((QObject*)but_default,
		   SIGNAL(clicked()),
		   this,
		   SLOT(slotDefault()));
  _putSettings();
}


dlgAccess::~dlgAccess(){
}


void dlgAccess::slotOk(){
  _getSettings();
  accept();
}


void dlgAccess::slotAbort(){
  reject();
}


void dlgAccess::slotDefault(){
  _rights=_default;
  _putSettings();
}


void dlgAccess::closeEvent(QCloseEvent *e){
  slotAbort();
}


void dlgAccess::_putSettings(){
  cbx_ur->setChecked(_rights & S_IRUSR);
  cbx_uw->setChecked(_rights & S_IWUSR);
  cbx_ux->setChecked(_rights & S_IXUSR);

  cbx_gr->setChecked(_rights & S_IRGRP);
  cbx_gw->setChecked(_rights & S_IWGRP);
  cbx_gx->setChecked(_rights & S_IXGRP);

  cbx_or->setChecked(_rights & S_IROTH);
  cbx_ow->setChecked(_rights & S_IWOTH);
  cbx_ox->setChecked(_rights & S_IXOTH);
}


void dlgAccess::_getSettings(){
  unsigned int r;

  r=0;
  if (cbx_ur->isChecked())
    r|=S_IRUSR;
  if (cbx_uw->isChecked())
    r|=S_IWUSR;
  if (cbx_ux->isChecked())
    r|=S_IXUSR;

  if (cbx_gr->isChecked())
    r|=S_IRGRP;
  if (cbx_gw->isChecked())
    r|=S_IWGRP;
  if (cbx_gx->isChecked())
    r|=S_IXGRP;

  if (cbx_or->isChecked())
    r|=S_IROTH;
  if (cbx_ow->isChecked())
    r|=S_IWOTH;
  if (cbx_ox->isChecked())
    r|=S_IXOTH;
  _rights=r;
}








