/***************************************************************************
 $RCSfile: dlgcheckreader.cpp,v $
 -------------------
 cvs         : $Id: dlgcheckreader.cpp,v 1.3 2003/02/17 04:19:40 aquamaniac Exp $
 begin       : Thu Dec 12 2002
 copyright   : (C) 2002 by Martin Preuss
 email       : martin@libchipcard.de


 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/






#include "dlgcheckreader.h"
#include <qstring.h>
#include <qtextview.h>
#include <qpushbutton.h>
#include <qlabel.h>



DlgCheckReader::DlgCheckReader(KProcess *proc,
			       QWidget* parent,
			       const char* name,
			       WFlags fl)
  :dlgCheckReaderBase(parent, name, fl)
  ,_process(proc)
{
  but_ok->setEnabled(false);
  but_abort->setEnabled(true);
  tv_output->setTextFormat(Qt::RichText);

  QObject::connect((QObject*)but_ok,
		   SIGNAL(clicked()),
		   this,
		   SLOT(close()));
  QObject::connect((QObject*)but_abort,
		   SIGNAL(clicked()),
		   this,
		   SLOT(slotAbort()));
  QObject::connect((QObject*)but_log,
		   SIGNAL(clicked()),
		   this,
		   SLOT(slotFullLog()));
  lab_result->setText("<font color=blue>Checking ...</font>");

  QObject::connect((QObject*)_process,
		   SIGNAL(processExited(KProcess*)),
		   this,
		   SLOT(slotProcessExited(KProcess*)));
  QObject::connect((QObject*)_process,
		   SIGNAL(receivedStdout(KProcess*,
					 char*,
					 int)),
		   (QObject*)this,
		   SLOT(slotReceivedStdOut(KProcess*,
					   char*,
					   int)));
  QObject::connect((QObject*)_process,
		   SIGNAL(receivedStderr(KProcess*,
					 char*,
					 int)),
		   (QObject*)this,
		   SLOT(slotReceivedStdErr(KProcess*,
					   char*,
					   int)));


}


DlgCheckReader::~DlgCheckReader(){
  delete _process;
}


void DlgCheckReader::finished(const QString &result){
  but_ok->setEnabled(true);
  lab_result->setText(result);
  but_abort->setEnabled(false);
}


void DlgCheckReader::slotProcessExited(KProcess *proc){
  emit signalProcessExited(this, proc);
}


void DlgCheckReader::slotReceivedStdOut(KProcess *proc,
					char *buffer,
					int buflen){
  QString ntxt;
  QString otxt;
  unsigned int i;

  ntxt.setLatin1(buffer, buflen);

  for (i=0; i<ntxt.length(); i++) {
    if (ntxt[i]=='\n')
      otxt+="<br>";
    else
      otxt+=ntxt[i];
  }
  tv_output->setText(tv_output->text()+otxt);
}


void DlgCheckReader::slotReceivedStdErr(KProcess *proc,
					char *buffer,
					int buflen){
  QString ntxt;
  QString otxt;
  unsigned int i;

  ntxt.setLatin1(buffer, buflen);

  otxt="<font color=red>";
  for (i=0; i<ntxt.length(); i++) {
    if (ntxt[i]=='\n')
      otxt+="<br>";
    else
      otxt+=ntxt[i];
  }
  otxt+="</font>";
  _errLog+=otxt;
}


void DlgCheckReader::slotAbort(){
  emit signalAborted(this, _process);
}


void DlgCheckReader::slotFullLog(){
  tv_output->setText(_errLog);
  but_log->setEnabled(false);
}




