/***************************************************************************
 $RCSfile: dlgservice.cpp,v $
                             -------------------
    cvs         : $Id: dlgservice.cpp,v 1.3 2003/02/17 04:19:40 aquamaniac Exp $
    begin       : Wed Oct 09 2002
    copyright   : (C) 2002 by Martin Preuss
    email       : martin@libchipcard.de

 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/


#include <qlineedit.h>
#include <qmessagebox.h>

#include "dlgservice.h"
#include "cfg_services.h"

#ifndef trUtf8
# define trUtf8 tr
#endif


dlgService::dlgService(int tp,
		       const string &addr,
		       unsigned int port,
		       QWidget* parent,
		       const char* name,
		       bool modal,
		       WFlags fl)
  :dlgServiceBase(parent,name,modal,fl)
  ,_changed(false)
  ,_addr(addr)
  ,_port(port)
  ,_type(tp)
{

  // connect all buttons
  QObject::connect((QObject*)but_abort,
		   SIGNAL(clicked()),
		   this,
		   SLOT(slotAbort()));
  QObject::connect((QObject*)but_ok,
		   SIGNAL(clicked()),
		   this,
		   SLOT(slotFinished()));
  QObject::connect((QObject*)ed_hostname,
		   SIGNAL(textChanged(const QString &)),
		   this,
		   SLOT(slotChanged(const QString &)));
  QObject::connect((QObject*)ed_port,
		   SIGNAL(textChanged(const QString &)),
		   this,
		   SLOT(slotChanged(const QString &)));
  QObject::connect((QObject*)cbo_type,
		   SIGNAL(activated(int )),
		   this,
		   SLOT(slotTypeChanged(int)));

}


dlgService::~dlgService(){
}


bool dlgService::_readStruct(int tp, const string &addr, unsigned int port){
  string tmp;

  cbo_type->setCurrentItem(tp);
  slotTypeChanged(tp);
  if (tp==CFG_SERVICE_TYPE_NET) {
    ed_hostname->setText(addr.c_str());
    ed_port->setText(QString::number(port));
  }
  else {
    ed_hostname->setText("");
    ed_port->setText("");
  }
  return true;
}


bool dlgService::_writeStruct(int &tp, string &address, unsigned int &port){
  int i;
  bool ok;

  // get type
  tp=cbo_type->currentItem();

  if (tp==CFG_SERVICE_TYPE_NET) {
    // get host
    address=ed_hostname->text().latin1();

    // get port
    i=ed_port->text().toInt(&ok,10);
    if (!ok || i>65535 || i<0) {
    QMessageBox::warning(this,
			 trUtf8("Edit Service"),
			 trUtf8("Bad port value. Must be 0-65535.\n"),
			 QMessageBox::Ok | QMessageBox::Default,
			 QMessageBox::NoButton,
			 QMessageBox::NoButton);
    return false;
    }
    port=i;
  }
  else {
    address.erase();
    port=0;
  }
  return true;
}


bool dlgService::init(){
  if (!_readStruct(_type, _addr, _port))
    return false;
  _changed=false;
  return true;
}


void dlgService::slotChanged(const QString &t){
  _changed=true;
}


void dlgService::slotTypeChanged(int i){
  _changed=true;
  ed_hostname->setEnabled(i!=CFG_SERVICE_TYPE_LOCAL);
  ed_port->setEnabled(i!=CFG_SERVICE_TYPE_LOCAL);
}


void dlgService::slotFinished(){
  string addr;
  unsigned int port;
  int tp;

  fprintf(stderr,"Finished.\n");

  // get changes
  if (_changed) {
    // is there an error within this terminal description ?
    if (!_writeStruct(tp, addr, port)) {
      // yes, don't allow to finish normally
      return;
    }
    _addr=addr;
    _port=port;
    _type=tp;
    // ok with changes
    fprintf(stderr,"Will accept.\n");
    accept();
    return;
  }
  // ok, but no changes
  reject();
}


void dlgService::slotAbort(){
  if (_changed) {
    switch(QMessageBox::warning(this,
				trUtf8("KCardSetup"),
				trUtf8("Configuration has changed\n"
				       "Do you want to abort anyway ?"),
				trUtf8("&Dismiss"),
				trUtf8("&Cancel"),0))
      {
      case 0: // yes
	break;

      case 1: // no
	return;
	break;

      default:
	return;
	break;
      } // switch
  } // if _changed

  // close with "reject"
  reject();
}


bool dlgService::service(int &tp, string &address, unsigned int &port){
  return _writeStruct(tp, address, port);
}



