/***************************************************************************
  $RCSfile: dlgterm.cpp,v $
                             -------------------
    cvs         : $Id: dlgterm.cpp,v 1.8 2003/02/17 18:44:11 aquamaniac Exp $
    begin       : Sun Mar 17 2002
    copyright   : (C) 2002 by Martin Preuss
    email       : martin@libchipcard.de


 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/

/*
 Changes

 */
#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <ctype.h>
#include <qdir.h>
#include <qmessagebox.h>
#include <qcombobox.h>
#include <qlineedit.h>
#include <qspinbox.h>
#include <qlabel.h>
#include <qlistview.h>
#include <qfiledialog.h>
#include <qcheckbox.h>
#include <kprocess.h>

#if KDE_GENERATION<3
# include <kapp.h>
#else
# include <kapplication.h>
#endif

#include "dlgterm.h"

#ifndef trUtf8
# define trUtf8 tr
#endif


list<string> dlgTerm::ctapiList;


dlgTerm::dlgTerm(CTREADERDESCRIPTION *config,
		 CTREADERDESCRIPTION *term,
		 CONFIGGROUP *drivers,
		 QWidget* parent,
		 const char* name,
		 bool modal,
		 WFlags fl)
  :dlgTermBase(parent,name,modal,fl)
,_readers(config)
  ,_reader(term)
  ,_drivers(drivers)
  ,_changed(false)
{
  _dlgScan=new dlgScanBase(this,0,WType_Modal);
  //_dlgScan->hide();

  // connect all buttons
  QObject::connect((QObject*)but_abort,
		   SIGNAL(clicked()),
		   this,
		   SLOT(slotAbort()));
  QObject::connect((QObject*)but_ok,
		   SIGNAL(clicked()),
		   this,
		   SLOT(slotFinished()));
  QObject::connect((QObject*)but_test,
		   SIGNAL(clicked()),
		   this,
		   SLOT(slotTest()));
  QObject::connect((QObject*)but_driver,
		   SIGNAL(clicked()),
		   this,
		   SLOT(slotFilename()));
  QObject::connect((QObject*)cbo_driver,
		   SIGNAL(activated(const QString &)),
		   this,
		   SLOT(slotChanged(const QString &)));
  QObject::connect((QObject*)cbo_driver,
		   SIGNAL(textChanged(const QString &)),
		   this,
		   SLOT(slotChanged(const QString &)));
  QObject::connect((QObject*)cbo_port,
		   SIGNAL(activated(const QString &)),
		   this,
		   SLOT(slotChanged(const QString &)));
  QObject::connect((QObject*)cbo_port,
		   SIGNAL(textChanged(const QString &)),
		   this,
		   SLOT(slotChanged(const QString &)));
  QObject::connect((QObject*)cbo_type,
		   SIGNAL(activated(const QString &)),
		   this,
		   SLOT(slotTypeChanged(const QString &)));
  QObject::connect((QObject*)cbo_type,
		   SIGNAL(textChanged(const QString &)),
		   this,
		   SLOT(slotChanged(const QString &)));
  QObject::connect((QObject*)ed_name,
		   SIGNAL(textChanged(const QString &)),
		   this,
		   SLOT(slotChanged(const QString &)));
  QObject::connect((QObject*)cbo_driver_type,
		   SIGNAL(activated(const QString &)),
		   this,
		   SLOT(slotChanged(const QString &)));
  QObject::connect((QObject*)cbx_haskeypad,
		   SIGNAL(stateChanged(int)),
		   this,
		   SLOT(slotChanged(int)));
  QObject::connect((QObject*)cbx_hasdisplay,
		   SIGNAL(stateChanged(int)),
		   this,
		   SLOT(slotChanged(int)));

}


dlgTerm::~dlgTerm(){
}


bool dlgTerm::_setReaderTypes() {
  CONFIGGROUP *g;

  cbo_type->clear();
  cbo_type->insertItem(tr("other"));
  g=_drivers;
  if (!g) {
    fprintf(stderr,"No drivers\n");
    return false;
  }
  g=g->groups;
  while(g) {
    cbo_type->insertItem(g->name);
    g=g->next;
  } // while
  return true;
}


bool dlgTerm::_setPorts(const QString &s) {
  CONFIGGROUP *g;
  CONFIGVARIABLE *v;

  cbo_port->clear();
  if (!_drivers) {
    fprintf(stderr,"No drivers\n");
    return false;
  }
  g=Config_GetGroup(_drivers, s.latin1(),CONFIGMODE_NAMEMUSTEXIST);
  if (!g) {
    fprintf(stderr,"Driver not found\n");
    return false;
  }
  g=Config_GetGroup(g,"ports",CONFIGMODE_NAMEMUSTEXIST);
  if (!g) {
    fprintf(stderr,"No ports found\n");
    return false;
  }
  v=g->variables;
  while(v) {
    cbo_port->insertItem(v->name);
    v=v->next;
  } // while
  return true;
}


bool dlgTerm::_addFile(const string &f){
  list<string>::iterator it;

  if (f.empty())
    return false;

  for (it=ctapiList.begin();
       it!=ctapiList.end();
       it++) {
    if (f==*it)
      return false;
  } // for
  ctapiList.push_back(f);
  return true;
}


void dlgTerm::_getFileList(const string &dir){
  CTPointer<QDir> d;

  _dlgScan->lab_scanning->setText(dir.c_str());
  qApp->processEvents();
  // create QDir
  d=new QDir(dir.c_str());

  // check if directory exists
  if (!d.ref().exists())
    // does not, nothing to do
    return;

  // otherwise set filter (we only want files)
  d.ref().setFilter( QDir::Files);
  d.ref().setSorting( QDir::Name);

  const QFileInfoList *list = d.ref().entryInfoList();
  QFileInfoListIterator it( *list );
  QFileInfo *fi;
  QListViewItem *item;

  while ( (fi=it.current()) ) {
    // for each file check if it matches one of the patterns
    if (d.ref().match(QString("libctapi*.so"),fi->fileName()) ||
	d.ref().match(QString("libctapi*.so.*"),fi->fileName()) ||
	d.ref().match(QString("libct_kaan*.so"),fi->fileName()) ||
	d.ref().match(QString("libct_kaan*.so.*"),fi->fileName()) ||
	d.ref().match(QString("libct_b1*.so"),fi->fileName()) ||
	d.ref().match(QString("libct_b1*.so.*"),fi->fileName()) ||
	d.ref().match(QString("libpcsclite.so.*"),fi->fileName()) ||

	/* NOTE: you can add here other filenames that you know
	 */

	d.ref().match(QString("libtowitoko*.so"),fi->fileName()) ||
	d.ref().match(QString("libtowitoko*.so.*"),fi->fileName())){
      // add file if it matches
      if (_addFile(fi->absFilePath().latin1())) {
	item=new QListViewItem(_dlgScan->list_drivers,
			       fi->absFilePath());
	qApp->processEvents();
      }
    }
    ++it;                               // next
  } // while
}


void dlgTerm::_getLibList() {
  if (ctapiList.empty()) {
    _dlgScan->list_drivers->clear();
    _dlgScan->lab_scanning->setText("");
    _dlgScan->show();

    _getFileList("/usr/local/lib");
    _getFileList("/usr/lib");
    _getFileList("/lib");
    _getFileList("/usr/local/towitoko/lib");
    _getFileList("/usr/local/towitoko-2.0.4/lib");
    _getFileList("/usr/local/towitoko-2.0.5/lib");
    _getFileList("/usr/local/towitoko-2.0.6/lib");
    _getFileList("/usr/local/towitoko-2.0.7/lib");
    _getFileList("/usr/local/lib/towitoko");
    _getFileList("/usr/towitoko");
    _getFileList("/usr/lib/towitoko");
    _getFileList("/usr/local/kobil");
    _getFileList("/usr/local/lib/kobil");
    _getFileList("/usr/kobil");
    _getFileList("/usr/lib/kobil");
    // add here other directories to scan

    _dlgScan->hide();
  }
}


bool dlgTerm::_compare(const string &s1, const string &s2){
  unsigned int i;

  if (s1.length()!=s2.length())
    return false;

  for (i=0; i<s1.length(); i++)
    if (toupper(s1[i])!=toupper(s2[i]))
      return false;
  return true;
}


bool dlgTerm::_selectComboText(QComboBox *b, const QString &t) {
  int i;
  int cs;

  cs=b->currentItem();
  for (i=0; i<b->count(); i++) {
    b->setCurrentItem(i);
    if (_compare(b->currentText().latin1(),t.latin1()))
      break;
  }
  if (i>=b->count()) {
    if (cs<b->count())
      b->setCurrentItem(cs);
    return false;
  }
  return true;
}


void dlgTerm::_setComboText(QComboBox *b, const QString &t) {
  // select it if it exists
  if (_selectComboText(b,t))
    return;
  // append it otherwise
  b->insertItem(t);
  // and THEN select it
  _selectComboText(b,t);
}


bool dlgTerm::_readStruct(CTREADERDESCRIPTION *ds){
  list<string>::reverse_iterator it;
  QString tmp;

  if (ds==0)
    return false;

  fprintf(stderr,"ctapilist::size: %d\n", ctapiList.size());
  if (ctapiList.size()<1)
    _getLibList();
  if (ctapiList.size()<1) {
    // no driver found
    QMessageBox::warning(this,
			 trUtf8("Edit Reader"),
			 trUtf8("Could not find card drivers\n"
				"You have to specify them manually."),
			 QMessageBox::Ok | QMessageBox::Default,
			 QMessageBox::NoButton,
			 QMessageBox::NoButton);
  } // if no driver
  else {
    // otherwise add drivers to combo box
    for (it=ctapiList.rbegin();
	 it!=ctapiList.rend();
	 it++){
      // insert driver to combo box
      cbo_driver->insertItem(QString((*it).c_str()));
    } // for
  } // if drivers

  // select driver
  _setComboText(cbo_driver,ds->driverName);

  // select driver type
  switch(ds->driverType) {
  case DriverTypePCSC:
    tmp="PCSC"; break;
  case DriverTypeCTAPI:
    tmp="CTAPI"; break;
  default:
    tmp="<unknown>"; break;
  } // switch
  _setComboText(cbo_driver_type,tmp);

  // name
  ed_name->setText(ds->name);

  // type
  _selectComboText(cbo_type,ds->typeName);
  _setPorts(ds->typeName);

  // port
  _selectComboText(cbo_port,ds->portName);

  // has keypad
  cbx_haskeypad->setChecked(ds->readerFlags & CHIPCARD_READERFLAGS_KEYPAD);

  // has display
  cbx_hasdisplay->setChecked(ds->readerFlags & CHIPCARD_READERFLAGS_DISPLAY);

  return true;
}


bool dlgTerm::_writeStruct(CTREADERDESCRIPTION *ds){
  QString str;
  unsigned int i;

  if (ctapiList.size()<1) {
    // no driver found
    QMessageBox::warning(this,
			 trUtf8("Edit Reader"),
			 trUtf8("You did not specify a card driver\n"
				"This terminal will not be accessable."),
			 QMessageBox::Ok | QMessageBox::Default,
			 QMessageBox::NoButton,
			 QMessageBox::NoButton);
  } // if no driver
  else {
    str=cbo_driver->currentText();
    if (str.length()>sizeof(ds->driverName)) {
      QMessageBox::warning(this,
			   trUtf8("Edit Reader"),
			   trUtf8("Driver name is too long\n"),
			   QMessageBox::Ok | QMessageBox::Default,
			   QMessageBox::NoButton,
			   QMessageBox::NoButton);
      return false;
    }
    strcpy(ds->driverName, str.latin1());
  }

  // driver type
  str=cbo_driver_type->currentText();
  if (!str) {
    QMessageBox::warning(this,
			 trUtf8("Edit Reader"),
			 trUtf8("You did not specify the driver type\n"
				"This terminal will not be accessable."),
			 QMessageBox::Ok | QMessageBox::Default,
			 QMessageBox::NoButton,
			 QMessageBox::NoButton);
    ds->driverType=DriverTypeUnknown;
  }
  else {
    if (str.upper()=="PCSC")
      ds->driverType=DriverTypePCSC;
    else if (str.upper()=="CTAPI")
      ds->driverType=DriverTypeCTAPI;
    else {
      QMessageBox::warning(this,
			   trUtf8("Edit Reader"),
			   trUtf8("You specified an unknown driver type\n"
				  "This terminal will not be accessable."),
			   QMessageBox::Ok | QMessageBox::Default,
			   QMessageBox::NoButton,
			   QMessageBox::NoButton);
      ds->driverType=DriverTypeUnknown;
    }
  }

  // name
  str=ed_name->text();
  if (str.length()>sizeof(ds->name)) {
    QMessageBox::warning(this,
			 trUtf8("Edit Reader"),
			 trUtf8("Reader name is too long\n"),
			 QMessageBox::Ok | QMessageBox::Default,
			 QMessageBox::NoButton,
			 QMessageBox::NoButton);
    return false;
  }
  strcpy(ds->name, str.latin1());

  // type
  str=cbo_type->currentText();
  if (!str) {
    QMessageBox::warning(this,
			 trUtf8("Edit Reader"),
			 trUtf8("You did not specify the type of the reader\n"
				"This terminal will not be accessable."),
			 QMessageBox::Ok | QMessageBox::Default,
			 QMessageBox::NoButton,
			 QMessageBox::NoButton);
    strcpy(ds->typeName,"other");
    return false;
  }
  else {
    if (str.length()+1>sizeof(ds->typeName)) {
      QMessageBox::critical(this,
			    trUtf8("Edit Reader"),
			    trUtf8("The reader type Name is too long.\n",
				   "This terminal will not be accessable."),
			    QMessageBox::Ok | QMessageBox::Default,
			    QMessageBox::NoButton,
			    QMessageBox::NoButton);
      strcpy(ds->typeName,"other");
      return false;
    }
    fprintf(stderr,"Typename: %s\n",str.latin1());
    strcpy(ds->typeName,str.latin1());
  }

  // port
  str=cbo_port->currentText();
  if (!str) {
    QMessageBox::warning(this,
			 trUtf8("Edit Reader"),
			 trUtf8("You did not specify the port\n"
				"This terminal may not be accessable."),
			 QMessageBox::Ok | QMessageBox::Default,
			 QMessageBox::NoButton,
			 QMessageBox::NoButton);
    strcpy(ds->portName, "0");
  }
  else {
    if (str.length()+1>sizeof(ds->portName)) {
      QMessageBox::critical(this,
			    trUtf8("Edit Reader"),
			    trUtf8("The port name is too long.\n",
				   "This terminal will not be accessable."),
			    QMessageBox::Ok | QMessageBox::Default,
			    QMessageBox::NoButton,
			    QMessageBox::NoButton);
      strcpy(ds->typeName,"0");
      return false;
    }
    strcpy(ds->portName,str.latin1());
  }

  // readerflags
  i=0;
  if (cbx_haskeypad->isChecked())
    i|=CHIPCARD_READERFLAGS_KEYPAD;
  if (cbx_hasdisplay->isChecked())
    i|=CHIPCARD_READERFLAGS_DISPLAY;

  ds->readerFlags=i;

  return true;
}


bool dlgTerm::init() {
  CTREADERDESCRIPTION *r;

  _setReaderTypes();
  if (!_readStruct(_reader))
    return false;

  r=_readers;
  while(r) {
    _addFile(r->driverName);
    r=r->next;
  } // while
  _changed=false;
  return true;
}


void dlgTerm::slotFinished(){
  // get changes
  if (_changed) {
    // is there an error within this terminal description ?
    if (!_writeStruct(_reader))
      // yes, don't allow to finish normally
      return;
    // temporary check for Cyberjack
  }

  // check for cyberjack, due to problems with "PERFORM_VERIFICATION"
#if 0 // problem fixed
  if ((_reader->readerType==ReaderTypeCyberjack) &&
      (_reader->readerFlags & CHIPCARD_READERFLAGS_KEYPAD))
    if (QMessageBox::warning(this,
			     trUtf8("Edit Reader"),
			     trUtf8("Please note that the "
				    "Reiner-SCT Cyberjack\n"
				    "has some problems with secure pin\n"
				    "verification.\n"
				    "So I recommend against using the "
				    "keypad of\n"
				    "this reader."),
			     trUtf8("Edit"),
			     trUtf8("Ignore"),
			     0,0,0)==0)
	return;
#endif

  // return
  if (_changed) {
    // ok with changes
    accept();
    return;
  }
  // ok, but no changes
  reject();
}


void dlgTerm::slotAbort(){
  if (_changed) {
    switch(QMessageBox::warning(this,
				trUtf8("QLCSetup"),
				trUtf8("Configuration has changed\n"
				       "Do you want to abort anyway ?"),
				trUtf8("&Dismiss"),
				trUtf8("&Cancel"),0))
      {
      case 0: // yes
	break;

      case 1: // no
	return;
	break;

      default:
	return;
	break;
      } // switch
  } // if _changed

  // close with "reject"
  reject();
}




void dlgTerm::slotTestAborted(DlgCheckReader *dlg,
			      KProcess *proc){
  QMessageBox::warning(this,
		       trUtf8("Test Reader"),
		       trUtf8("Test aborted by user"),
		       QMessageBox::Ok | QMessageBox::Default,
		       QMessageBox::NoButton,
		       QMessageBox::NoButton);
  dlg->close();
}


void dlgTerm::slotTestExited(DlgCheckReader *dlg,
			     KProcess *proc){
  if (proc->normalExit()) {
    switch(proc->exitStatus()) {
    case 0: // ok
      dlg->finished("<font color=green>Reader found ;-)</font>");
      break;
    case 1: // bad parameters
      dlg->finished(trUtf8("<font color=red>"
			   "Bad parameters"
			   "</font>"));
      break;
    case 2: // internal error
      dlg->finished(trUtf8("<font color=red>"
			   "Internal error, please see log :-("
			   "</font>"));
      break;
    case 3: // no reader
      dlg->finished(trUtf8("<font color=red>"
			   "Reader not found :-/"
			   "</font>"));
      break;
    default:
      dlg->finished(trUtf8("<font color=red>"
			   "Unknown error, should never happen :-{"
			   "</font>"));
      break;
    } // switch
  } // if normalExit
  else {
    dlg->finished(trUtf8("<font color=red>"
			 "Check-program died unexpectedly:-{"
			 "</font>"));
  }
}


void dlgTerm::slotTest(){
  CTREADERDESCRIPTION *rs;
  DlgCheckReader *dlg;
  KProcess *proc;
  bool b;
  CONFIGGROUP *cfg;
  int rv;
  QString tmp;

  rs=CTCore_ReaderDescr_new();
  if (!_writeStruct(rs)) {
    fprintf(stderr,"Bad settings, please edit\n");
    CTCore_ReaderDescr_free(rs);
    return;
  }

  cfg=Config_new();
  rv=CTCore_WriteReaderDescr(cfg,rs);
  if (rv) {
    QMessageBox::critical(this,
			  trUtf8("Edit Reader"),
			  trUtf8("Could not transform "
				 "reader description (1).\n"
				 "This is an internal error, "
				 "please report to\n"
				 "bug@libchipcard.de\n"),
			  QMessageBox::Ok | QMessageBox::Default,
			  QMessageBox::NoButton,
			  QMessageBox::NoButton);
    fprintf(stderr,"Could not create config from reader description.\n");
    CTCore_ReaderDescr_free(rs);
    Config_free(cfg);
    return;
  }
  CTCore_ReaderDescr_free(rs);

  proc=new KProcess();
  proc->setRunPrivileged(false);
  *proc << "readertest";
  tmp=Config_GetValue(cfg, "port","",0);
  if (tmp.isEmpty()) {
    fprintf(stderr,"Empty port.\n");
    Config_free(cfg);
    return;
  }
  *proc << "-p" << tmp;
  tmp=Config_GetValue(cfg, "readertype","",0);
  if (tmp.isEmpty()) {
    fprintf(stderr,"Empty readertype.\n");
    Config_free(cfg);
    return;
  }
  *proc << "-r" << tmp;
  tmp=Config_GetValue(cfg, "drivertype","",0);
  if (tmp.isEmpty()) {
    fprintf(stderr,"Empty drivertype.\n");
    Config_free(cfg);
    return;
  }
  *proc << "-d" << tmp;;
  *proc << "-D" << QString(rs->driverName);
  Config_free(cfg);

  dlg=new DlgCheckReader(proc, this,0,
			 Qt::WType_Modal|WDestructiveClose);

  QObject::connect((QObject*)dlg,
		   SIGNAL(signalProcessExited(DlgCheckReader*, KProcess*)),
		   (QObject*)this,
		   SLOT(slotTestExited(DlgCheckReader*, KProcess*)));
  QObject::connect((QObject*)dlg,
		   SIGNAL(signalAborted(DlgCheckReader*, KProcess*)),
		   (QObject*)this,
		   SLOT(slotTestAborted(DlgCheckReader*, KProcess*)));

  dlg->show();

  // now start the process
  b=proc->start(KProcess::NotifyOnExit,
		KProcess::AllOutput);
  if (!b) {
    QMessageBox::warning(this,
			 trUtf8("Test Reader"),
			 trUtf8("<qt>Unable to start check-program.<br>"
				"Please make sure that the tool<br>"
				"<font color=blue>readertest</font> "
				"is in reachable by PATH.<br>"
				"You can do this by installing that tool to "
				"a default directory like "
				"<i>/usr/local/bin</i><br>"),
			 QMessageBox::Ok | QMessageBox::Default,
			 QMessageBox::NoButton,
			 QMessageBox::NoButton);
    dlg->close();
  }
}



void dlgTerm::closeEvent(QCloseEvent *e){
  slotAbort();
}


void dlgTerm::slotChanged(const QString &t){
  _changed=true;
}


void dlgTerm::slotTypeChanged(const QString &t){
  QString s;

  s=cbo_port->currentText();
  _setPorts(t);
  _selectComboText(cbo_port, s);
  _changed=true;
}


void dlgTerm::slotChanged(int){
  _changed=true;
}


void dlgTerm::slotFilename() {
  int i;

  // get file name
  QString f=QFileDialog::getOpenFileName(QString::null,
					 trUtf8("Drivers (*.so *.so.*)"),
					 this,
					 0,
					 trUtf8("Select Driver"));
  if (!f)
    return;

  // add filename to driver list and to listview
  if (_addFile(f.latin1()))
    cbo_driver->insertItem(f);

  // select driver
  for (i=0; i<cbo_driver->count(); i++) {
    cbo_driver->setCurrentItem(i);
    if (cbo_driver->currentText()==f)
      break;
  }
  if (i>=cbo_driver->count()){
    _changed=true;
    cbo_driver->setCurrentItem(0);
  }
}


bool dlgTerm::saveReader(CTREADERDESCRIPTION *ts){
  fprintf(stderr," Will save reader \"%s\"\n",ts->name);
  return _writeStruct(ts);
}


