/***************************************************************************
  $RCSfile: main.cpp,v $
                             -------------------
    cvs         : $Id: main.cpp,v 1.7 2003/02/18 00:42:12 aquamaniac Exp $
    begin       : Sat Mar 16 2002
    copyright   : (C) 2002 by Martin Preuss
    email       : martin@libchipcard.de


 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "version.h"
#include "chipcard.h"

#if KDE_GENERATION<3
# include <kapp.h>
#else
# include <kapplication.h>
#endif

#include <kcmdlineargs.h>
#include <kaboutdata.h>
#include <klocale.h>

#include "configdialog.h"
#include <qmessagebox.h>

#include <chameleon/directory.h>
#include <sys/types.h>
#include <signal.h>
#include <errno.h>
#include <string.h>


#ifndef trUtf8
# define trUtf8 tr
#endif

using namespace std;


static const char *description =
    I18N_NOOP("Tool to setup your chip card readers");
	
	
static KCmdLineOptions options[] ={
    { 0, 0, 0 }
    // INSERT YOUR COMMANDLINE OPTIONS HERE
};



bool checkProcess(int pid, const char *n) {
  FILE *f;
  char nbuffer[256];
  char fbuffer[512];
  int i;
  string tmp;

  sprintf(nbuffer,"/proc/%d/cmdline",pid);
  f=fopen(nbuffer,"r");
  if (!f)
    return false;
  i=fread(fbuffer,1,sizeof(fbuffer)-1,f);
  if (i<1) {
    fclose(f);
    return false;
  }
  fclose(f);
  fbuffer[i]=0;
  tmp=string(fbuffer);
  if (tmp.find(n)!=string::npos)
    return true;
  return false;
}


bool killallProcesses(const char *n, int sig) {
  DIRECTORYDATA *d;
  char nbuffer[256];
  int pid;
  bool killed;

  killed=false;
  d=Directory_new();
  if (Directory_Open(d,"/proc")) {
    Directory_free(d);
    return 0;
  }
  while (!Directory_Read(d,
			 nbuffer,
			 sizeof(nbuffer))) {
    pid=atoi(nbuffer);
    if (pid!=0) {
      if (checkProcess(pid, n)) {
	if (kill(pid,sig)) {
	  fprintf(stderr,"Could not send signal to process %d (%s)\n",
		  pid,
		  strerror(errno));
	}
	else {
	  fprintf(stderr,"Signal send to process %d\n",pid);
	  killed=true;
	}
      }
    }
  } /* while */
  return killed;
}



int main(int argc, char *argv[]){
    KAboutData aboutData("kcardsetup", I18N_NOOP("kCardSetup"),
			 KCARDSETUP_VERSION_STRING,
			 description,
			 KAboutData::License_GPL,
			 "(c) 2003, Martin Preuss",
			 0,
			 0,
			 "martin@libchipcard.de");
    aboutData.addAuthor("Martin Preuss",0, "martin@libchipcard.de");
    KCmdLineArgs::init( argc, argv, &aboutData );
    KCmdLineArgs::addCmdLineOptions( options ); // Add our own options.

    KApplication a;
    ConfigDialog *dlg;
    bool daemonSuspended;
    int rv;

    fprintf(stderr,"Looking for daemon, suspending it\n");
    daemonSuspended=killallProcesses("chipcardd", SIGTSTP);
    if (daemonSuspended) {
      fprintf(stderr,"Daemon suspended, we can work now ;-)\n");
    }
    else {
      fprintf(stderr,"Daemon not suspended.\n");
    }

    dlg=new ConfigDialog(0,0,
			 Qt::WType_TopLevel|Qt::WStyle_ContextHelp);
    a.setMainWidget(dlg);
    if (!dlg->init())
	return 2;
    dlg->show();
#if QT_GENERATION<3
    rv=0;
#else
    if (daemonSuspended) {
      QMessageBox::information(dlg,
			       QWidget::trUtf8("KCardSetup"),
			       QWidget::trUtf8("The chipcard daemon is suspended"
					       "\n"
					       "It will be resumed as soon as "
					       "KCardSetup has finished."),
			       QMessageBox::Ok | QMessageBox::Default,
			       QMessageBox::NoButton,
			       QMessageBox::NoButton);
    }
    rv=a.exec();
#endif
    if (daemonSuspended) {
      fprintf(stderr,"Looking for daemon, resuming it\n");
      if (killallProcesses("chipcardd", SIGCONT)) {
	fprintf(stderr,"Daemon resumed.\n");
      }
      else {
	fprintf(stderr,"Daemon not resumed.\n");
      }
    }
    return rv;
}





