/***************************************************************************
 $RCSfile: dlgkvk.cpp,v $
                             -------------------
    cvs         : $Id: dlgkvk.cpp,v 1.2 2002/12/28 04:15:38 aquamaniac Exp $
    begin       : Wed Dec 26 2001
    copyright   : (C) 2001 by Martin Preuss
    email       : martin@libchipcard.de

 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/


#include <qmessagebox.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qdatetime.h>
#include <qcombobox.h>
#include "dlgkvk.h"
#include "chipcard.h"


#ifndef trUtf8
# define trUtf8 tr
#endif


bool dlgKVK::init(){
  return true;
}


dlgKVK::dlgKVK(QWidget* parent,
               const char* name,
               bool modal,
               WFlags fl)
    :dlgKVKBase(parent,name,fl)
{
  QObject::connect((QObject*)but_read,SIGNAL(clicked()),this,SLOT(slotRead()));
  QObject::connect((QObject*)but_exit,SIGNAL(clicked()),this,SLOT(close()));
}


dlgKVK::~dlgKVK(){
}


bool dlgKVK::errorCheck(const string &title,
                        const string &errstr,
                        CTError err,
                        unsigned char ad) {
    if (!err.isOk(ad)) {
        QMessageBox mbox(title.c_str(),
                         (errstr+"\n("+err.errorString()+")").c_str(),
                         QMessageBox::Warning,
                         QMessageBox::Ok | QMessageBox::Default,
                         QMessageBox::NoButton,
                         QMessageBox::NoButton);
        mbox.exec();
        return false;
    }
    return true;
}


void dlgKVK::slotRead(){
    insuranceData data;
    string tmp;
    QDate d;
    CTError err;
    CTPointer<CTKVKCard> card;
    int rv;
    unsigned int rflags;
    unsigned int rstatus;
    int requestid;

    card=new CTKVKCard();
    fprintf(stderr,"\n------------------------------------\n"
	    "KMedicalCard: will check if card is inserted:\n");

    rv=card.ref().startWaitForCard(requestid,
				   false,
				   0,
				   0,
				   CHIPCARD_STATUS_INSERTED,
				   CHIPCARD_STATUS_INSERTED |
				   CHIPCARD_STATUS_PROCESSOR);
    if (rv!=CHIPCARD_SUCCESS) {
      fprintf(stderr, "1:Could not mount card: %d\n",rv);
      return;
    }

    // check with a short timeout
    rv=card.ref().checkWaitForCard(requestid,2,rflags,rstatus);
    if (rv!=CHIPCARD_SUCCESS &&
	rv!=CHIPCARD_ERROR_NO_MESSAGE) {
      fprintf(stderr, "2:Could not mount card: %d\n",rv);
      card.ref().stopWaitForCard(requestid);
      return;
    }

    if (rv!=CHIPCARD_SUCCESS) {
      // no card inserted, ask user to do so
      fprintf(stderr,"Card is not inserted, asking user.\n");
      QMessageBox mbox(trUtf8("Chipcard"),
		       trUtf8("Please insert your card."),
		       QMessageBox::Information,
		       QMessageBox::Ok | QMessageBox::Default,
		       QMessageBox::Abort | QMessageBox::Escape,
		       QMessageBox::NoButton);
      if (mbox.exec()!=QMessageBox::Ok){
	card.ref().stopWaitForCard(requestid);
	return;
      }
    }

    while(1) {
      if (!(rstatus & CHIPCARD_STATUS_INSERTED)) {
	// now check with a longer timeout
	fprintf(stderr,"Checking again.\n");
	rv=card.ref().checkWaitForCard(requestid,20,rflags,rstatus);
	if (rv!=CHIPCARD_SUCCESS) {
	  if (rv!=CHIPCARD_ERROR_NO_MESSAGE) {
	    card.ref().stopWaitForCard(requestid);
	    fprintf(stderr, "3:Could not mount card: %d\n",rv);
	    return;
	  }
	}
      }
      else
	rv=CHIPCARD_SUCCESS;

      if (rv==CHIPCARD_SUCCESS) {
	card.ref().stopWaitForCard(requestid);
	fprintf(stderr,"\n------------------------------------\n"
		"KMedicalCard: will open card:\n");
	err=card.ref().openCard();
	if (!errorCheck(trUtf8("Chipcard").latin1(),
			trUtf8("Could not open card, maybe there is no card inserted ?").latin1(),
			err)){
	  fprintf(stderr,
		  "KMedicalCard: could not open card:\n%s\n",
		  err.errorString().c_str());
	  return;
	}

	if (card.ref().isProcessorCard()) {
	  QMessageBox mbox(trUtf8("Chipcard").latin1(),
			   trUtf8("Chip card is not a German health insurance card.").latin1(),
			   QMessageBox::Warning,
			   QMessageBox::Ok | QMessageBox::Default,
			   QMessageBox::NoButton,
			   QMessageBox::NoButton);
	  mbox.exec();
	  card.ref().closeCard();
	  return;
	}

	fprintf(stderr,"\n------------------------------------\n"
		"KMedicalCard: will read card data:\n");
	err=card.ref().readCardData(data);
	if (!errorCheck(trUtf8("Chipcard").latin1(),
			trUtf8("Could not read card data.").latin1(),
			err)) {
	  fprintf(stderr,
		  "KMedicalCard: could not read card data:\n%s\n",
		  err.errorString().c_str());
	  card.ref().closeCard();
	  return;
	}
	if (!data.isValid) {
	  fprintf(stderr,
		  "KMedicalCard: card data is invalid\n%s\n",
		  err.errorString().c_str());
	  QMessageBox mbox(trUtf8("Chipcard").latin1(),
			   trUtf8("Chip card is not a German health insurance card.").latin1(),
			   QMessageBox::Warning,
			   QMessageBox::Ok | QMessageBox::Default,
			   QMessageBox::NoButton,
			   QMessageBox::NoButton);
	  mbox.exec();
	  card.ref().closeCard();
	  return;
	}

	fprintf(stderr,"\n------------------------------------\n"
		"KMedicalCard: will close card:\n");
	err=card.ref().closeCard();
	if (!errorCheck(trUtf8("Chipcard").latin1(),
			trUtf8("Could not close card.").latin1(),
			err,0x62)){
	  return;
	}

	// show data
	lab_companyname->setText(data.insuranceCompanyName.c_str());
	lab_companycode->setText(data.insuranceCompanyCode.c_str());
	lab_insnumber->setText(data.insuranceNumber.c_str());
	lab_insstate->setText(data.insuranceState.c_str());
	lab_title->setText(data.title.c_str());
	lab_forename->setText(data.forename.c_str());
	tmp=data.nameSuffix+data.name;
	lab_name->setText(tmp.c_str());
	if (data.dateOfBirth.length()<8)
	  lab_birthdate->setText(data.dateOfBirth.c_str());
	else {
	  d=QDate(atoi(data.dateOfBirth.substr(4,4).c_str()),
		  atoi(data.dateOfBirth.substr(2,2).c_str()),
		  atoi(data.dateOfBirth.substr(0,2).c_str()));
	  lab_birthdate->setText(d.toString());
	}
	lab_street->setText(data.addrStreet.c_str());
	lab_state->setText(data.addrState.c_str());
	lab_zip->setText(data.addrPostalCode.c_str());
	lab_city->setText(data.addrCity.c_str());
	lab_bestbefore->setText(data.bestBefore.c_str());
	lab_number->setText(data.cardNumber.c_str());
	if (data.eastOrWest=="1")
	  lab_west->setText(trUtf8("Western Germany"));
	else
	  lab_west->setText(trUtf8("Eastern Germany"));
	lab_isvalid->setText(data.isValid?trUtf8("Yes"):trUtf8("No"));
	return;
      }
    } // while
}


