/***************************************************************************
  $RCSfile: dlgpcsc.cpp,v $
                             -------------------
    cvs         : $Id: dlgpcsc.cpp,v 1.3 2003/01/05 11:26:26 aquamaniac Exp $
    begin       : Tue Aug 06 2002
    copyright   : (C) 2002 by Martin Preuss
    email       : martin@libchipcard.de


 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/

#include <qlistview.h>
#include <qmessagebox.h>
#include <qpushbutton.h>
#include <qlabel.h>

#include "dlgpcsc.h"
#include "dlgreader.h"

#ifndef trUtf8
# define trUtf8 tr
#endif


dlgPCSC::dlgPCSC(const string &cfgfilename,
                 QWidget* parent,
                 const char* name,
                 WFlags fl)
    :dlgPCSCBase(parent,name,fl)
    ,_cfgFileName(cfgfilename)
    ,_changed(false)
{
    // connect all buttons
    QObject::connect((QObject*)but_add,
                     SIGNAL(clicked()),
                     this,
                     SLOT(slotAdd()));
    QObject::connect((QObject*)but_remove,
                     SIGNAL(clicked()),
                     this,
                     SLOT(slotRemove()));
    QObject::connect((QObject*)but_edit,
                     SIGNAL(clicked()),
                     this,
                     SLOT(slotEdit()));
    QObject::connect((QObject*)but_copy,
                     SIGNAL(clicked()),
                     this,
                     SLOT(slotCopy()));
    QObject::connect((QObject*)but_abort,
                     SIGNAL(clicked()),
                     this,
                     SLOT(slotAbort()));
    QObject::connect((QObject*)but_save,
                     SIGNAL(clicked()),
                     this,
                     SLOT(slotSave()));
    QObject::connect((QObject*)but_finished,
                     SIGNAL(clicked()),
                     this,
                     SLOT(slotFinished()));
    but_save->setEnabled(false);

}


dlgPCSC::~dlgPCSC(){
}


bool dlgPCSC::init(){
    CTError err;

    err=_cfg.readFile(_cfgFileName);
    if (!err.isOk()) {
        fprintf(stderr,
                "Could not read config file \"%s\"\n%s\n",
                _cfgFileName.c_str(),
                err.errorString().c_str());
        if (QMessageBox::warning(0,
                                 trUtf8("Chipcard"),
                                 ("Could not load configuration file \""+_cfgFileName
                                  +"\"\nDo you want to create a new one ?").c_str(),
                                 trUtf8("&Create"),trUtf8("&Don't Create"),0,
                                 0,1)!=0)
            return false;
    }

    updateList();
    return true;

}


string dlgPCSC::_makeNextName(const string &name) {
    unsigned int i;
    int num;
    string body;
    char buffer[16];

    num=0;
    body=name;

    // first get the number at the end of this name
    for (i=name.length()-1;
         i>=0;
         i--) {
        // is there still a digit ?
        if (name.at(i)<'0' || name.at(i)>'9')
            // no
            break;
    } // for

    // valid pos ?
    if (i>=0 && i<name.length()-1){
        // yes, read the number, create the body
        num=atoi((name.substr(i+1)).c_str());
        body=name.substr(0,i+1);
    }

    // increment number
    num++;

    // create new name and return
    snprintf(buffer,sizeof(buffer),"%d",num);
    body+=buffer;
    return body;
}


ReaderEntry *dlgPCSC::currentReader(){
    QListViewItem *item;

    item=list_readers->currentItem();
    if (!item)
        return 0;
    return _cfg.findReader(item->text(0).latin1());
}


void dlgPCSC::updateList(){
    list<ReaderEntry>::reverse_iterator iter;
    QListViewItem *item;

    list_readers->clear();

    // add all terminals to the list
    for (iter=_cfg.readers().rbegin();
         iter!=_cfg.readers().rend();
         iter++) {
        item=new QListViewItem(list_readers,
                               (*iter).friendlyName().c_str(),
			       (*iter).channelId().c_str(),
                               trUtf8((*iter).libPath().c_str())
                              );
    } // for
}


void dlgPCSC::setChanged(bool ch){
    _changed=ch;
    but_save->setEnabled(ch);
}


void dlgPCSC::slotAdd(){
    string name;
    int i;
    ReaderEntry reader;

    // try to create a unique name 20 times
    name="Terminal";
    i=20;
    while(i) {
        name=_makeNextName(name);
        if (!(_cfg.findReader(name)))
            break;
        i--;
    }

    if (!i) {
        // could not find a unique name within 20 tries
        QMessageBox::critical(this,
                              trUtf8("Add reader"),
                              trUtf8("Could not create a unique name\n"
                                     "This is an internal error, please report to\n"
                                     "martin@libchipcard.de"),
                              QMessageBox::Ok | QMessageBox::Default,
                              QMessageBox::NoButton,
                              QMessageBox::NoButton);
        return;
    }

    // init new struct
    reader.setFriendlyName(name);
    reader.setDeviceName("DEVICE_"+name);
    reader.setChannelId("0x01");
    // add new struct to list
    _cfg.addReader(reader);
    setChanged(true);
    updateList();
}


void dlgPCSC::slotRemove(){
    ReaderEntry *ts;

    // get currently selected reader
    ts=currentReader();
    if (!ts) {
        QMessageBox::critical(this,
                              trUtf8("Remove Reader"),
                              trUtf8("You must first select a device"),
                              QMessageBox::Ok | QMessageBox::Default,
                              QMessageBox::NoButton,
                              QMessageBox::NoButton);
        return;
    }

    // ask the user
    if (QMessageBox::warning(this,
                             trUtf8("Remove Reader"),
                             trUtf8("Really remove this device ?"),
                             QMessageBox::Ok | QMessageBox::Default,
                             QMessageBox::Abort | QMessageBox::Escape,
                             QMessageBox::NoButton)!=QMessageBox::Ok)
        return;

    // remove the terminal
    if (!_cfg.removeReader(ts->friendlyName()))
        QMessageBox::critical(this,
                              trUtf8("Remove Reader"),
                              trUtf8("Could not remove reader"),
                              QMessageBox::Ok | QMessageBox::Default,
                              QMessageBox::NoButton,
                              QMessageBox::NoButton);
    setChanged(true);
    updateList();
}


void dlgPCSC::slotEdit(){
    ReaderEntry *ts;
    CTPointer<dlgReader> dlgt;
    int answer;

    // get currently selected reader
    ts=currentReader();
    if (!ts) {
        QMessageBox::critical(this,
                              trUtf8("Edit Reader"),
                              trUtf8("You must first select a device"),
                              QMessageBox::Ok | QMessageBox::Default,
                              QMessageBox::NoButton,
                              QMessageBox::NoButton);
        return;
    }
    dlgt=new dlgReader(&_cfg,ts,this,0,true);
    if (!dlgt.ref().init())
        return;
    answer=dlgt.ref().exec();
    if (answer==QDialog::Accepted) {
        setChanged(true);
        updateList();
    }
}


void dlgPCSC::slotCopy(){
    ReaderEntry *ts;
    ReaderEntry ts2;
    string name;
    int i;

    // get currently selected reader
    ts=currentReader();
    if (!ts) {
        QMessageBox::critical(this,
                              trUtf8("Copy reader"),
                              trUtf8("You must first select a device"),
                              QMessageBox::Ok | QMessageBox::Default,
                              QMessageBox::NoButton,
                              QMessageBox::NoButton);
        return;
    }
    // copy the definition
    ts2=*ts;
    name=ts2.friendlyName();

    // try to create a unique name 20 times
    i=20;
    while(i) {
        name=_makeNextName(name);
        if (!(_cfg.findReader(name)))
            break;
        i--;
    }

    // could not find a unique name within 20 tries
    if (!i) {
        QMessageBox::critical(this,
                              trUtf8("Copy reader"),
                              trUtf8("Could not create a unique name\n"
                                     "This is an internal error, please report to\n"
                                     "martin@libchipcard.de"),
                              QMessageBox::Ok | QMessageBox::Default,
                              QMessageBox::NoButton,
                              QMessageBox::NoButton);
        return;
    }
    ts2.setFriendlyName(name);
    _cfg.addReader(ts2);
    setChanged(true);
    updateList();
}


void dlgPCSC::slotAbort(){
    CTError err;

    if (_changed) {
        switch(QMessageBox::warning(this,
                                    trUtf8("QLCSetup"),
                                    trUtf8("Configuration has changed\n"
                                           "Do you want to save it now ?"),
                                    trUtf8("&Save"),
                                    trUtf8("&Don't Save"),
                                    trUtf8("&Cancel"),
                                    0,2))
        {
        case 0: // save
            // save data
            err=_cfg.writeFile(_cfgFileName);
            if (!err.isOk()) {
                fprintf(stderr,"Error writing file:\n%s\n",
                        err.errorString().c_str());
                if (QMessageBox::critical(this,
                                          trUtf8("QLCSetup"),
                                          trUtf8("Could not write configuration\n"
                                                 "Do you want to exit anyway ?"),
                                          trUtf8("&Exit"),
                                          trUtf8("&Don't Exit"),
                                          0, 1,0)!=0)
                    return;
            }
            setChanged(false);
            break;

        case 1: // don't save
            break;

        case 2: // cancel
            return;

        } // switch
    } // if changed

    // close application
    setChanged(false);
    close();
}


void dlgPCSC::slotSave(){
    CTError err;

    err=_cfg.writeFile(_cfgFileName);
    if (!err.isOk()) {
        fprintf(stderr,"Error writing file:\n%s\n",
                err.errorString().c_str());
        QMessageBox::critical(this,
                              trUtf8("QLCSetup"),
                              trUtf8("Could not write configuration"),
                              trUtf8("&Ok"),0,0);
        return;
    }
    setChanged(false);
}


bool dlgPCSC::prepareExit() {
    CTError err;

    if (_changed) {
        switch(QMessageBox::warning(this,
                                    trUtf8("QLCSetup"),
                                    trUtf8("Configuration has changed\n"
                                           "Do you want to save it now ?"),
                                    trUtf8("&Save"),
                                    trUtf8("&Don't Save"),
                                    trUtf8("&Cancel"),
                                    0,2))
        {
        case 0: // save
            // save data
            err=_cfg.writeFile(_cfgFileName);
            if (!err.isOk()) {
                fprintf(stderr,"Error writing file:\n%s\n",
                        err.errorString().c_str());
                if (QMessageBox::critical(this,
                                          trUtf8("QLCSetup"),
                                          trUtf8("Could not write configuration\n"
                                                 "Do you want to exit anyway ?"),
                                          trUtf8("&Exit"),
                                          trUtf8("&Abort")
                                          ,0,1,0)!=0)
                    return false;
            }
            setChanged(false);
            break;

        case 1: // don't save
            break;

        case 2: // cancel
            return false;

        default:
            break;
        } // switch
    } // if _changed

    setChanged(false);
    return true;
}


void dlgPCSC::slotFinished(){
    if (prepareExit())
        // close application
        close();
}


void dlgPCSC::closeEvent(QCloseEvent *e){
    if (prepareExit())
        e->accept();
}




