/***************************************************************************
  $RCSfile: dlgreader.cpp,v $
                             -------------------
    cvs         : $Id: dlgreader.cpp,v 1.2 2002/12/28 04:15:39 aquamaniac Exp $
    begin       : Tue Aug 06 2002
    copyright   : (C) 2002 by Martin Preuss
    email       : martin@libchipcard.de


 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/

#include <qmessagebox.h>
#include <qfiledialog.h>
#include <qlineedit.h>


#include "dlgreader.h"

#ifndef trUtf8
# define trUtf8 tr
#endif


dlgReader::dlgReader(ReaderConf *cfg,
                     ReaderEntry *reader,
                     QWidget* parent,
                     const char* name,
                     bool modal,
                     WFlags fl)
    :dlgReaderBase(parent,name,modal,fl)
    ,_changed(false)
    ,_cfg(cfg)
    ,_reader(reader)
{
    // connect all buttons
    QObject::connect((QObject*)but_abort,
                     SIGNAL(clicked()),
                     this,
                     SLOT(slotAbort()));
    QObject::connect((QObject*)but_ok,
                     SIGNAL(clicked()),
                     this,
                     SLOT(slotFinished()));
    QObject::connect((QObject*)but_file,
                     SIGNAL(clicked()),
                     this,
                     SLOT(slotFilename()));
    QObject::connect((QObject*)ed_friendlyname,
                     SIGNAL(textChanged(const QString &)),
                     this,
                     SLOT(slotChanged(const QString &)));
    QObject::connect((QObject*)ed_devicename,
                     SIGNAL(textChanged(const QString &)),
                     this,
                     SLOT(slotChanged(const QString &)));
    QObject::connect((QObject*)ed_libpath,
                     SIGNAL(textChanged(const QString &)),
                     this,
                     SLOT(slotChanged(const QString &)));

}


dlgReader::~dlgReader(){
}


bool dlgReader::init(){
    bool retval;

    if (!_cfg || !_reader) {
        QMessageBox::critical(this,
                              trUtf8("Edit Reader"),
                              trUtf8("Either configuration or reader "
                                     "(or even both) not given.\n"
                                     "This is an internal error\n"
                                     "Please report to martin@libchipcard.de"),
                              QMessageBox::Ok | QMessageBox::Default,
                              QMessageBox::NoButton,
                              QMessageBox::NoButton);
        return false;
    } // if no ts
    retval=_readStruct();
    _changed=false;
    return retval;
}


void dlgReader::slotFinished(){
    ReaderEntry ts;

    // get changes
    if (_changed) {
        // is there an error within this terminal description ?
        if (!_writeStruct(ts))
            // yes, don't allow to finish normally
            return;
        *_reader=ts;
        // ok with changes
        accept();
        return;
    }
    // ok, but no changes
    reject();
}


void dlgReader::slotAbort(){
    if (_changed) {
        switch(QMessageBox::warning(this,
                                    trUtf8("QLCSetup"),
                                    trUtf8("Configuration has changed\n"
                                           "Do you want to abort anyway ?"),
                                    trUtf8("&Dismiss"),
                                    trUtf8("&Cancel"),0))
        {
        case 0: // yes
            break;

        case 1: // no
            return;
            break;

        default:
            return;
            break;
        } // switch
    } // if _changed

    // close with "reject"
    reject();
}


void dlgReader::slotChanged(const QString &t){
    _changed=true;
}


void dlgReader::closeEvent(QCloseEvent *e){
    slotAbort();
}


void dlgReader::slotFilename() {
    int i;

    // get file name
    QString f=QFileDialog::getOpenFileName(QString::null,
                                           trUtf8("Drivers (*.so *.so.*)"),
                                           this,
                                           0,
                                           trUtf8("Select Driver"));
    if (!f)
        return;

    // add filename to driver list and to listview
    ed_libpath->setText(f);
}


bool dlgReader::_selectComboText(QComboBox *b, const string &t) {
    int i;

    // port
    for (i=0; i<b->count(); i++) {
        b->setCurrentItem(i);
        if (CTMisc::compare(b->currentText().latin1(),t.c_str()))
            break;
    }
    if (i>=b->count()) {
        b->setCurrentItem(0);
        return false;
    }
    return true;
}


void dlgReader::_setComboText(QComboBox *b, const string &t) {
    // select it if it exists
    if (_selectComboText(b,t))
        return;
    // append it otherwise
    b->insertItem(t.c_str());
    // and THEN select it
    _selectComboText(b,t);
}


bool dlgReader::_readStruct(){
    ed_friendlyname->setText(_reader->friendlyName().c_str());
    ed_devicename->setText(_reader->deviceName().c_str());
    ed_libpath->setText(_reader->libPath().c_str());
    _setComboText(cbo_port,_reader->channelId());
    return true;
}


bool dlgReader::_writeStruct(ReaderEntry &ts){
    ReaderEntry *r;

    ts.setFriendlyName(ed_friendlyname->text().latin1());
    ts.setDeviceName(ed_devicename->text().latin1());
    ts.setLibPath(ed_libpath->text().latin1());
    ts.setChannelId(cbo_port->currentText().latin1());

    // now check the members
    if (ts.friendlyName().empty()) {
        QMessageBox::warning(this,
                             trUtf8("Edit Reader"),
                             trUtf8("You did not specify the friendly "
                                    "name of the reader.\n"
                                    "This terminal would not be accessible."),
                             QMessageBox::Ok | QMessageBox::Default,
                             QMessageBox::NoButton,
                             QMessageBox::NoButton);
        return false;
    }
    r=_cfg->findReader(ts.friendlyName());
    if (r && r!=_reader) {
        QMessageBox::critical(this,
                              trUtf8("Edit Reader"),
                              trUtf8("The friendly name MUST be unique !\n"
                                     "Otherwise some terminals will not "
                                     "be accessable."),
                              QMessageBox::Ok | QMessageBox::Default,
                              QMessageBox::NoButton,
                              QMessageBox::NoButton);
        return false;
    }
    if (ts.deviceName().empty()) {
        QMessageBox::warning(this,
                             trUtf8("Edit Reader"),
                             trUtf8("You did not specify the device "
                                    "name of the reader.\n"
                                    "This terminal would not be accessible."),
                             QMessageBox::Ok | QMessageBox::Default,
                             QMessageBox::NoButton,
                             QMessageBox::NoButton);
        return false;
    }
    if (ts.libPath().empty()) {
        QMessageBox::warning(this,
                             trUtf8("Edit Reader"),
                             trUtf8("You did not specify the library path.\n"
                                    "This terminal would not be accessible."),
                             QMessageBox::Ok | QMessageBox::Default,
                             QMessageBox::NoButton,
                             QMessageBox::NoButton);
        return false;
    }
    if (ts.channelId().empty()) {
        QMessageBox::warning(this,
                             trUtf8("Edit Reader"),
                             trUtf8("You did not specify the channel ID "
                                    "of the reader.\n"
                                    "This terminal would not be accessible."),
                             QMessageBox::Ok | QMessageBox::Default,
                             QMessageBox::NoButton,
                             QMessageBox::NoButton);
        return false;
    }

    return true;
}





