/***************************************************************************
  $RCSfile: readerconf.cpp,v $
                             -------------------
    cvs         : $Id: readerconf.cpp,v 1.2 2002/12/28 04:15:39 aquamaniac Exp $
    begin       : Tue Aug 06 2002
    copyright   : (C) 2002 by Martin Preuss
    email       : martin@libchipcard.de


 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/

#include <errno.h>
#include <ctype.h>

#include "readerconf.h"



ReaderEntry::ReaderEntry(){
}


ReaderEntry::~ReaderEntry(){
}


string ReaderEntry::dump(){
    string result;

    result+="Reader\n";
    result+="------------------------------\n";
    result+="Friendly Name: ";
    result+=_friendlyName+"\n";
    result+="Device Name  : ";
    result+=_deviceName+"\n";
    result+="Library Path : ";
    result+=_libPath+"\n";
    result+="ChannelId    : ";
    result+=_channelId+"\n";

    return result;
}




ReaderConf::ReaderConf(){
}


ReaderConf::~ReaderConf(){
}


bool ReaderConf::_postprocess(string &s){
    unsigned int i;

    i=0;
    // skip leading blanks
    while(i<s.length()) {
        if (s.at(i)>=33)
            break;
        i++;
    }
    if (i)
        s.erase(0,i);

    // skip blanks at end of string
    for (i=s.length()-1; i>=0; i--)
        if (s[i]>32)
            break;
    if (i>=0)
        s.erase(i+1);

    // remove quotation marks
    if (s.at(0)=='\"' && s.at(s.length()-1)=='\"')
        s=s.substr(1,s.length()-2);
    else if (s.at(0)=='\'' && s.at(s.length()-1)=='\'')
        s=s.substr(1,s.length()-2);
    return true;
}


bool ReaderConf::_compare(const string &s1, const string &s2){
    unsigned int i;

    if (s1.length()!=s2.length())
        return false;

    for (i=0; i<s1.length(); i++)
        if (toupper(s1[i])!=toupper(s2[i]))
            return false;
    return true;
}


CTError ReaderConf::_readConfig(FILE *f){
    char buffer[1024];
    string line;
    unsigned int pos1;
    unsigned int pos2;
    string n1;
    string n2;
    char *c;
    bool have1;
    ReaderEntry current;

    have1=false;
    while(!feof(f)) {
        // fill buffer
        fgets(buffer,sizeof(buffer),f);
        // postprocess buffer
        c=buffer;
        // find end
        while(*c) {
            if (*c=='\n' || *c=='#') {
                *c=0;
            }
            c++;
        }
        line=buffer;

        // skip blanks at the beginning
        pos1=0;
        while (pos1<line.length()) {
            if (line[pos1]>32)
                break;
            pos1++;
        } // while
        if (pos1<line.length())
            line.erase(0,pos1);

        if (!line.empty()) {
            // find end of the name
            pos1=0;
            while (pos1<line.length()) {
                if (line[pos1]<33)
                    break;
                pos1++;
            } // while
            if (pos1>=line.length()) {
                return CTError("ReaderConf::_readConfig()",
                               k_CTERROR_INVALID,
                               0,
                               0,
                               "no argument");
            }

            // get name
            n1=line.substr(0,pos1);
            if (!_postprocess(n1))
                return CTError("ReaderConf::_readConfig()",
                               k_CTERROR_INVALID,
                               0,
                               0,
                               "error in name",
                               n1);

            // get value
            if (pos1+1<line.length()) {
                n2=line.substr(pos1+1);
                if (!_postprocess(n2))
                    return CTError("ReaderConf::_readConfig()",
                                   k_CTERROR_INVALID,
                                   0,
                                   0,
                                   "error in argument",
                                   n2);
            }
            else {
                return CTError("ReaderConf::_readConfig()",
                               k_CTERROR_INVALID,
                               0,
                               0,
                               "no argument (2)");
            }

            if (_compare(n1,"friendlyname")) {
                if (have1) {
                    _readers.push_back(current);
                }
                current=ReaderEntry();
                have1=true;
                current.setFriendlyName(n2);
            }
            else if (_compare(n1,"devicename")) {
                have1=true;
                current.setDeviceName(n2);
            }
            else if (_compare(n1,"libpath")) {
                have1=true;
                current.setLibPath(n2);
            }
            else if (_compare(n1,"channelid")) {
                have1=true;
                current.setChannelId(n2);
            }
            else {
                return CTError("ReaderConf::_readConfig()",
                               k_CTERROR_INVALID,
                               0,
                               0,
                               "unknown variable",
                               line);
            }
        } // if line !empty
    } // while !feof
    if (have1) {
        _readers.push_back(current);
    }

    return CTError();
}


CTError ReaderConf::readFile(const string &n){
    FILE *f;
    CTError res;

    f=fopen(n.c_str(),"r");
    if (f==0)
        return CTError("ReaderConf::readFile()",
                       k_CTERROR_INVALID,
                       0,
                       0,
                       "could not open file "+n,
                       strerror(errno));
    res=_readConfig(f);
    if (fclose(f))
        return CTError("ReaderConf::readFile()",
                       k_CTERROR_INVALID,
                       0,
                       0,
                       "could not close file "+n,
                       strerror(errno));
    return res;
}


CTError ReaderConf::_writeConfig(FILE *f){
    list<ReaderEntry>::iterator it;

    fprintf(f,"# Configuration file for pcsc-lite\n");
    fprintf(f,"# automatically created by KLCSetup (LibChipCard)\n");
    fprintf(f,"\n");
    for (it=_readers.begin();
         it!=_readers.end();
         it++) {
        fprintf(f,"FRIENDLYNAME %s\n",(*it).friendlyName().c_str());
        fprintf(f,"DEVICENAME %s\n",(*it).deviceName().c_str());
        fprintf(f,"LIBPATH %s\n",(*it).libPath().c_str());
        fprintf(f,"CHANNELID %s\n",(*it).channelId().c_str());
        fprintf(f,"\n");
    } // for
    return CTError();
}


CTError ReaderConf::writeFile(const string &n){
    FILE *f;
    CTError res;

    f=fopen(n.c_str(),"w+");
    if (f==0)
        return CTError("ReaderConf::writeFile()",
                       k_CTERROR_INVALID,
                       0,
                       0,
                       "could not open file "+n,
                       strerror(errno));
    res=_writeConfig(f);
    if (fclose(f))
        return CTError("ReaderConf::writeFile()",
                       k_CTERROR_INVALID,
                       0,
                       0,
                       "could not close file "+n,
                       strerror(errno));
    return res;
}


void ReaderConf::addReader(const ReaderEntry &e){
    _readers.push_back(e);
}


void ReaderConf::clear(){
    _readers.clear();
}


string ReaderConf::dump(){
    list<ReaderEntry>::iterator it;
    string result;

    for (it=_readers.begin();
         it!=_readers.end();
         it++) {
        result+=(*it).dump()+="\n";
    } // for

    return result;
}


/*
list<CTTermSection> ReaderConf::asTermSectionList(){
    list<CTTermSection> tsl;
    list<ReaderEntry>::iterator it;
    string result;
    CTTermSection ts;
    int i;
    int tnum;

    tnum=0;
    for (it=_readers.begin();
         it!=_readers.end();
         it++) {
        ts=CTTermSection();
        ts.setDriver(WINSCARD_LIB);
        ts.setDriverType("pcsc");
        i=CTMisc::string2num((*it).channelId(),"%i");
        switch (i) {
        case 1: ts.setPort("COM1"); break;
        case 2: ts.setPort("COM2"); break;
        case 3: ts.setPort("COM3"); break;
        case 4: ts.setPort("COM4"); break;
        default: ts.setPort((*it).channelId());
        } // switch
        ts.setShortName("pcsc"+CTMisc::num2string(tnum++));
        ts.setSystemName((*it).friendlyName()+" 0 0");
        ts.setLongName((*it).friendlyName());
        tsl.push_back(ts);
    } // for

    return tsl;
}
*/


ReaderEntry *ReaderConf::findReader(const string &fname){
    list<ReaderEntry>::iterator it;

    for (it=_readers.begin();
         it!=_readers.end();
         it++) {
        if (CTMisc::compare((*it).friendlyName(),fname))
            return &(*it);
    } // for

    return 0;
}


bool ReaderConf::removeReader(const string &fname){
    list<ReaderEntry>::iterator it;

    for (it=_readers.begin();
         it!=_readers.end();
         it++) {
        if (CTMisc::compare((*it).friendlyName(),fname)) {
            _readers.erase(it);
            return true;
        }
    } // for
    return false;
}




