/***************************************************************************
  $RCSfile: readerconf.h,v $
                             -------------------
    cvs         : $Id: readerconf.h,v 1.2 2002/12/28 04:15:39 aquamaniac Exp $
    begin       : Tue Aug 06 2002
    copyright   : (C) 2002 by Martin Preuss
    email       : martin@libchipcard.de


 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef READERCONF_H
#define READERCONF_H

#include <stdio.h>
#include <string>
#include <list>

#include <chipcard.h>


/**
 *
 */
class ReaderEntry {
private:
    string _friendlyName;
    string _deviceName;
    string _libPath;
    string _channelId;

public:
    ReaderEntry();
    ~ReaderEntry();

    string friendlyName() const { return _friendlyName;};
    void setFriendlyName(const string &n) { _friendlyName=n;};
    string deviceName() const { return _deviceName;};
    void setDeviceName(const string &n) { _deviceName=n;};
    string libPath() const { return _libPath;};
    void setLibPath(const string &n) { _libPath=n;};
    string channelId() const { return _channelId;};
    void setChannelId(const string &n) {_channelId=n;};

    string dump();
};

/**
 *
 */
class ReaderConf {
private:
    list<ReaderEntry> _readers;

    bool _postprocess(string &s);
    bool _compare(const string &s1, const string &s2);
    CTError _readConfig(FILE *f);
    CTError _writeConfig(FILE *f);

public:
    ReaderConf();
    ~ReaderConf();
    list<ReaderEntry> &readers() { return _readers;};
    CTError readFile(const string &n);
    CTError writeFile(const string &n);
    void addReader(const ReaderEntry &e);
    bool removeReader(const string &fname);
    void clear();
    string dump();
    //list<CTTermSection> asTermSectionList();
    ReaderEntry *findReader(const string &fname);
};



#endif // READERCONF_H


